# -*- coding: utf-8 -*-
#
# $Date: 2013-03-15 17:48:48 +0900 (金, 15 3 2013) $
# $Rev: 69 $
#
import wx

import threading
import time

EVT_WORKER_FINISHED_ID = wx.NewId()

def EVT_WORKER_FINISHED(win, func):
    win.Connect(-1, -1, EVT_WORKER_FINISHED_ID, func)
    return

class WorkerThread(threading.Thread):
    def __init__(self, workerMethod, notifyReceiver = None):
        threading.Thread.__init__(self)
        self.__workerMethod = workerMethod
        self.__notifyReceiver = notifyReceiver
        return

    def run(self):
        self.__workerMethod()

        if self.__notifyReceiver != None:
            event = wx.PyEvent()
            event.SetEventType(EVT_WORKER_FINISHED_ID)
            wx.PostEvent(self.__notifyReceiver, event)
            pass
        return

EVT_WATCHER_UPDATE_ID = wx.NewId()

def EVT_WATCHER_UPDATE(win, func):
    win.Connect(-1, -1, EVT_WATCHER_UPDATE_ID, func)
    return

class WorkerWatcher(threading.Thread):
    def __init__(self, workerMethod, updateReceiver = None, notifyReceiver = None):
        threading.Thread.__init__(self)
        self.__workerMethod = workerMethod
        self.__updateReceiver = updateReceiver
        self.__notifyReceiver = notifyReceiver
        return

    def run(self):
        worker = WorkerThread(self.__workerMethod, self.__notifyReceiver)
        worker.start()

        event = wx.PyEvent()
        event.SetEventType(EVT_WATCHER_UPDATE_ID)

        while worker.isAlive():
            if self.__updateReceiver != None:
                wx.PostEvent(self.__updateReceiver, event)
                pass

            time.sleep(0.1)
            pass
        return

import xrc.base
import VTKPanel
import qspace
import vector
import dialog
import define

class MainFrame(xrc.base.FrameBase):
    def __init__(self, mode):
        xrc.base.FrameBase.__init__(self, 'resources/MainFrame.xrc', 'MainFrame')

        self.__mode = mode
        self.__isMeasurementDataLoaded = False

        self.__qPointSize = 0.01

        panel = self.getControl('VtkWindow')

        self.__vtkPanel = VTKPanel.VTKPanel(panel)

        panel.GetSizer().Add(self.__vtkPanel)
        panel.Bind(wx.EVT_SIZE, self.resize)

        listControl = self.getControl('QpListCtrl')
        listControl.Bind(wx.EVT_LIST_COL_CLICK, self.lstColumnClick)
        listControl.Bind(wx.EVT_LIST_ITEM_SELECTED, self.lstSelected)
        listControl.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.lstDeselected)
        listControl.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.lstActivated)
        listControl.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.lstRightClicked)

        self.__qSpaceManager = qspace.QSpaceManager(mode)

        self.__createMenuBar()
        self.__createToolBar()

        self.__lstSortMode = 'IJK'
        self.__updateQPointList()
        self.UpdateView()

        self.Fit()

        self.__rotationControlDialog = dialog.RotationControlDialog(self)

        self.__vtkPanel.CreateQEGraphPlane()
        self.__vtkPanel.CreateQRegionGraphPlane()

        EVT_WORKER_FINISHED(self, self.WorkerFinished)
        return

    def resize(self, event):
        self.__vtkPanel.SetSize(event.GetSize())
        return

    def __createMenuBar(self):
        menuBar = wx.MenuBar()
        self.SetMenuBar(menuBar)

        self.__createFileMenu(menuBar)
        self.__createViewMenu(menuBar)
        self.__createDisplayControlMenu(menuBar)
        self.__createRegionMenu(menuBar)
        self.__createControlMenu(menuBar)
        self.__createGraphMenu(menuBar)
        return

    def __createFileMenu(self, menuBar):
        self.__fileMenu = wx.Menu()

        menuItem = self.__fileMenu.Append(wx.ID_ANY, 'Sample Information')
        self.Bind(wx.EVT_MENU, self.PopupSampleInformationDialog, menuItem)

        self.__fileMenu.AppendSeparator()

        menuItem = self.__fileMenu.Append(wx.ID_ANY, 'Load method file')
        self.Bind(wx.EVT_MENU, self.MethodFileAccess, menuItem)

        menuItem = self.__fileMenu.Append(wx.ID_ANY, 'Save method file')
        self.Bind(wx.EVT_MENU, self.MethodFileAccess, menuItem)

        self.__fileMenu.AppendSeparator()

        import platform

        if platform.system() == 'Windows':
            menuItem = self.__fileMenu.Append(wx.ID_ANY, 'Load NeXus file')
            self.Bind(wx.EVT_MENU, self.LoadDataFile, menuItem)
            pass

        menuItem = self.__fileMenu.Append(wx.ID_ANY, 'Load Boost file')
        self.Bind(wx.EVT_MENU, self.LoadDataFile, menuItem)

        self.__fileMenu.AppendSeparator()

        menuItem = self.__fileMenu.Append(wx.ID_ANY, 'Exit')
        self.Bind(wx.EVT_MENU, self.close, menuItem)

        menuBar.Append(self.__fileMenu, 'File')
        return

    def __createViewMenu(self, menuBar):
        self.__viewMenu = wx.Menu()

        menuItem = self.__viewMenu.Append(wx.ID_ANY, 'qx view\tCtrl-X')
        self.Bind(wx.EVT_MENU, self.ViewControl, menuItem)
        menuItem = self.__viewMenu.Append(wx.ID_ANY, 'qy view\tCtrl-Y')
        self.Bind(wx.EVT_MENU, self.ViewControl, menuItem)
        menuItem = self.__viewMenu.Append(wx.ID_ANY, 'qz view\tCtrl-Z')
        self.Bind(wx.EVT_MENU, self.ViewControl, menuItem)

        self.__viewMenu.AppendSeparator()

        menuItem = self.__viewMenu.Append(wx.ID_ANY, 'qa view')
        self.Bind(wx.EVT_MENU, self.ViewControl, menuItem)
        menuItem = self.__viewMenu.Append(wx.ID_ANY, 'qb view')
        self.Bind(wx.EVT_MENU, self.ViewControl, menuItem)
        menuItem = self.__viewMenu.Append(wx.ID_ANY, 'qc view')
        self.Bind(wx.EVT_MENU, self.ViewControl, menuItem)

        self.__viewMenu.AppendSeparator()

        menuItem = self.__viewMenu.AppendRadioItem(wx.ID_ANY, 'Perspective view')
        self.Bind(wx.EVT_MENU, self.ProjectionControl, menuItem)
        menuItem = self.__viewMenu.AppendRadioItem(wx.ID_ANY, 'Parallel-projection view')
        self.Bind(wx.EVT_MENU, self.ProjectionControl, menuItem)

        menuBar.Append(self.__viewMenu, 'View')
        return

    def IsMeasurementDataLoaded(self):
        return self.__isMeasurementDataLoaded

    def GetBounds(self):
        return self.__qSpaceManager.GetBounds()

    def ViewControl(self, event):
        item = self.__viewMenu.FindItemById(event.GetId())
        if item == None:
            item = self.__toolBar.FindById(event.GetId())
            operation = item.GetShortHelp()
            pass
        else:
            operation = item.GetLabel()
            pass

        bounds = self.GetBounds()

        if operation == 'qx view':
            self.__vtkPanel.CreateCameraView('qx', self.__qSpaceManager.GetBoundingSize(), self.__qSpaceManager.GetCenter(), bounds[1], bounds[3], bounds[5])
            pass
        elif operation == 'qy view':
            self.__vtkPanel.CreateCameraView('qy', self.__qSpaceManager.GetBoundingSize(), self.__qSpaceManager.GetCenter(), bounds[1], bounds[3], bounds[5])
            pass
        elif operation == 'qz view':
            self.__vtkPanel.CreateCameraView('qz', self.__qSpaceManager.GetBoundingSize(), self.__qSpaceManager.GetCenter(), bounds[1], bounds[3], bounds[5])
            pass
        else:
            import math

            xw = bounds[1] - bounds[0]
            yw = bounds[3] - bounds[2]
            zw = bounds[5] - bounds[4]
            scale = math.sqrt(xw * xw + yw * yw + zw * zw)

            if operation == 'qa view':
                self.__vtkPanel.CreateLatticeView('qa', self.__qSpaceManager.ltc.ra_rotate,
                                               self.__qSpaceManager.ltc.rb_rotate,
                                               self.__qSpaceManager.ltc.rc_rotate, scale)
                pass
            elif operation == 'qb view':
                self.__vtkPanel.CreateLatticeView('qb', self.__qSpaceManager.ltc.ra_rotate,
                                               self.__qSpaceManager.ltc.rb_rotate,
                                               self.__qSpaceManager.ltc.rc_rotate, scale)
                pass
            elif operation == 'qc view':
                self.__vtkPanel.CreateLatticeView('qc', self.__qSpaceManager.ltc.ra_rotate,
                                               self.__qSpaceManager.ltc.rb_rotate,
                                               self.__qSpaceManager.ltc.rc_rotate, scale)
                pass
            pass

        self.__vtkPanel.Render()
        return

    def ProjectionControl(self, event):
        item = self.__viewMenu.FindItemById(event.GetId())
        if item == None:
            item = self.__toolBar.FindById(event.GetId())
            operation = item.GetShortHelp()
            pass
        else:
            operation = item.GetLabel()
            pass

        if operation == 'Perspective view':
            self.__vtkPanel.ProjectionControl(False)
            pass
        else:
            self.__vtkPanel.ProjectionControl(True)
            pass
        return

    def __createDisplayControlMenu(self, menuBar):
        self.__displayControlMenu = wx.Menu()

        self.__regionSurfaceControlMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show region surface')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__regionSurfaceControlMenuItem)
        self.__contourLineMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show Et contour line')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__contourLineMenuItem)
        self.__contourLineMenuItem.Enable(False)
        self.__etIsosurfaceControlMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show Et isosurface')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__etIsosurfaceControlMenuItem)
        self.__qPointDisplayMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show Q points')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__qPointDisplayMenuItem)
        self.__elasticScatteringLineMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show elastic-scattering line')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__elasticScatteringLineMenuItem)
        self.__cellDisplayMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show cell')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__cellDisplayMenuItem)
        self.__rotationGuideControlMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show rotation guide')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__rotationGuideControlMenuItem)
        self.__pixelIsosurfaceControlMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show pixel isosurface')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__pixelIsosurfaceControlMenuItem)
        self.__latticeAxesControlMenuItem = self.__displayControlMenu.AppendCheckItem(wx.ID_ANY, 'Show lattice axes')
        self.Bind(wx.EVT_MENU, self.DisplayControl, self.__latticeAxesControlMenuItem)

        psdMenu = wx.Menu()
        self.__displayControlMenu.AppendSubMenu(psdMenu, 'Show PSDs ...')

        self.__regionNameMap = dict()
        self.__PSDMenuItems = []

        for name in self.__qSpaceManager.param.GetAllRegionNames():
            subMenu = wx.Menu()
            psdMenu.AppendSubMenu(subMenu, name)

            regionProperty = self.__qSpaceManager.param.detectorProps[name]
            for i in xrange(regionProperty.nPSD):
                psdNumber = i + regionProperty.PSD0
                menuItem = subMenu.AppendCheckItem(wx.ID_ANY, str(psdNumber))
                self.Bind(wx.EVT_MENU, self.ShowPSD, menuItem)
                self.__regionNameMap[menuItem.GetId()] = name

                self.__PSDMenuItems.append(menuItem)
                pass
            pass

        psdMenu.AppendSeparator()
        menuItem = psdMenu.Append(wx.ID_ANY, 'Reset all check')
        self.Bind(wx.EVT_MENU, self.ResetPSDSelection, menuItem)

        self.__displayControlMenu.AppendSeparator()

        menuItem = self.__displayControlMenu.Append(wx.ID_ANY, 'Object settings...\tCtrl-O')
        self.Bind(wx.EVT_MENU, self.PopupObjectSettingDialog, menuItem)

        menuBar.Append(self.__displayControlMenu, '3D_objects')
        return

    def DisplayControl(self, event):
        menuItemLabel = self.__displayControlMenu.FindItemById(event.GetId()).GetLabel()

        if menuItemLabel == self.__regionSurfaceControlMenuItem.GetLabel() or menuItemLabel == self.__contourLineMenuItem.GetLabel():
            self.__RegionSurfaceControl()
            pass
        elif menuItemLabel == self.__etIsosurfaceControlMenuItem.GetLabel():
            self.__EtIsosurfaceControl()
            pass
        elif menuItemLabel == self.__qPointDisplayMenuItem.GetLabel():
            self.__vtkPanel.ShowQPoint(self.__qPointDisplayMenuItem.IsChecked())
            pass
        elif menuItemLabel == self.__elasticScatteringLineMenuItem.GetLabel():
            self.__ElasticScatteringLineControl()
            pass
        elif menuItemLabel == self.__cellDisplayMenuItem.GetLabel():
            self.__CellDisplayControl()
            pass
        elif menuItemLabel == self.__rotationGuideControlMenuItem.GetLabel():
            self.__RotationGuideControl()
            pass
        elif menuItemLabel == self.__pixelIsosurfaceControlMenuItem.GetLabel():
            self.__PixelIsosurfaceControl()
            pass
        elif menuItemLabel == self.__latticeAxesControlMenuItem.GetLabel():
            self.__LatticeAxesControl()
            pass

        self.__vtkPanel.Render()
        return

    def PopupSampleInformationDialog(self, event):
        dlgSample = dialog.SampleInformationDialog(self)

        if dlgSample.ShowModal() == wx.ID_OK:
            sampleName = dlgSample.GetSampleName()
            latticeSystem = dlgSample.GetLatticeSystem()
            latticeSystemKind = dlgSample.GetLatticeSystemKind()

            self.SetParameter(qspace.TAG_SAMPLE_LTCSYS, latticeSystem)
            self.SetParameter(qspace.TAG_SAMPLE_LTCKIND, latticeSystemKind)
            self.SetParameter(qspace.TAG_SAMPLE_NAME, sampleName)

            parameters = dlgSample.GetInputParameters()

            import decimal

            for name in parameters.keys():
                self.SetParameter(name, decimal.Decimal(parameters[name]))
                pass

            a = self.GetParameter(qspace.TAG_SAMPLE_A)
            alpha = self.GetParameter(qspace.TAG_SAMPLE_ALPHA)

            if latticeSystem == qspace.Lattice.LT_SYS_TRICLINIC:
                pass # Nothing to do
            elif latticeSystem == qspace.Lattice.LT_SYS_MONOCLINIC:
                self.SetParameter(qspace.TAG_SAMPLE_ALPHA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_GAMMA, decimal.Decimal(90))
                pass
            elif latticeSystem == qspace.Lattice.LT_SYS_ORTHORHOMBIC:
                self.SetParameter(qspace.TAG_SAMPLE_ALPHA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_BETA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_GAMMA, decimal.Decimal(90))
                pass
            elif latticeSystem == qspace.Lattice.LT_SYS_TETRAGONAL:
                self.SetParameter(qspace.TAG_SAMPLE_B, a)
                self.SetParameter(qspace.TAG_SAMPLE_ALPHA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_BETA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_GAMMA, decimal.Decimal(90))
                pass
            elif latticeSystem == qspace.Lattice.LT_SYS_TRIGONAL:
                self.SetParameter(qspace.TAG_SAMPLE_B, a)
                self.SetParameter(qspace.TAG_SAMPLE_C, a)
                self.SetParameter(qspace.TAG_SAMPLE_BETA, alpha)
                self.SetParameter(qspace.TAG_SAMPLE_GAMMA, alpha)
                pass
            elif latticeSystem == qspace.Lattice.LT_SYS_HEXAGONAL:
                self.SetParameter(qspace.TAG_SAMPLE_B, a)
                self.SetParameter(qspace.TAG_SAMPLE_ALPHA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_BETA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_GAMMA, decimal.Decimal(120))
                pass
            elif latticeSystem == qspace.Lattice.LT_SYS_CUBIC:
                self.SetParameter(qspace.TAG_SAMPLE_B, a)
                self.SetParameter(qspace.TAG_SAMPLE_C, a)
                self.SetParameter(qspace.TAG_SAMPLE_ALPHA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_BETA, decimal.Decimal(90))
                self.SetParameter(qspace.TAG_SAMPLE_GAMMA, decimal.Decimal(90))
                pass

            self.UpdateSampleInformation()
            pass
        return

    def ResetPSDSelection(self, event):
        for menuItem in self.__PSDMenuItems:
            if menuItem.IsChecked():
                menuItem.Check(False)
                regionName = self.__regionNameMap[menuItem.GetId()]
                psdNumber = int(menuItem.GetLabel())
                self.__vtkPanel.ShowPSDOutline(regionName, psdNumber, False)
                pass
            pass
        return

    def PopupObjectSettingDialog(self, event):
        dlg = dialog.ObjectSettingDialog(self)
        dlg.Show()
        return

    def ApplySettings(self):
        self.__progressBarDialog = dialog.ProgressBarDialog(self, 'Please wait for update view', 0)
        # self.__progressBarDialog.Show()

        self.__workerMode = 'update view'

        # EVT_WATCHER_UPDATE(self.__progressBarDialog, self.__progressBarDialog.Update)

        # WorkerWatcher(self.UpdateView, self.__progressBarDialog, self).start()

        self.UpdateView()

        self.__vtkPanel.ShowQPoint(self.__qPointDisplayMenuItem.IsChecked())

        qPointList = self.__qSpaceManager.GetVisibleQPoints()
        self.__vtkPanel.SetQPointColor(qPointList)
        self.__vtkPanel.SetQPointOpacity(qPointList)

        self.__vtkPanel.ShowQPoint(self.__qPointDisplayMenuItem.IsChecked())

        self.__UpdateRegionDisplay()
        return

    def __RegionSurfaceControl(self):
        regionNames = self.GetCheckedRegions()

        paintSurface = self.__regionSurfaceControlMenuItem.IsChecked()
        drawContour = self.__contourLineMenuItem.IsChecked()

        self.__contourLineMenuItem.Enable(paintSurface)

        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            try:
                regionNames.index(regionName)
                self.__vtkPanel.ShowDetectorSurface(regionName, paintSurface, drawContour)
                pass
            except ValueError:
                self.__vtkPanel.ShowDetectorSurface(regionName, False, drawContour)
                pass
            pass
        return

    def __EtIsosurfaceControl(self):
        regionNames = self.GetCheckedRegions()

        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            try:
                regionNames.index(regionName)
                self.__vtkPanel.ShowEtIsosurface(regionName, self.__etIsosurfaceControlMenuItem.IsChecked())
                pass
            except ValueError:
                self.__vtkPanel.ShowEtIsosurface(regionName, False)
                pass
            pass
        return

    def __ElasticScatteringLineControl(self):
        regionNames = self.GetCheckedRegions()

        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            try:
                regionNames.index(regionName)
                self.__vtkPanel.ShowElasticScatteringLine(regionName, self.__elasticScatteringLineMenuItem.IsChecked())
                pass
            except ValueError:
                self.__vtkPanel.ShowElasticScatteringLine(regionName, False)
                pass
            pass
        return

    def __CellDisplayControl(self):
        self.__vtkPanel.ShowCell(self.__cellDisplayMenuItem.IsChecked())
        return

    def __RotationGuideControl(self):
        listControl = self.getControl('QpListCtrl')
        item = listControl.GetFirstSelected()

        if item == -1:
            return

        selectedQPointKey = listControl.GetItemText(item)

        import math

        axisType = self.GetParameter(qspace.TAG_ROTATE_GUIDEAXIS)
        psi = math.radians(self.GetParameter(qspace.TAG_ROTATE_PSI))
        chi = math.radians(self.GetParameter(qspace.TAG_ROTATE_CHI))
        phi = math.radians(self.GetParameter(qspace.TAG_ROTATE_PHI))

        self.__vtkPanel.SetRotationGuide(axisType, psi, chi, phi,
                                         self.__qSpaceManager.ltc.ra_orig,
                                         self.__qSpaceManager.ltc.rb_orig,
                                         self.__qSpaceManager.ltc.rc_orig,
                                         self.__qSpaceManager.GetQPoint(selectedQPointKey))

        self.__vtkPanel.ShowRotationGuide(self.__rotationGuideControlMenuItem.IsChecked())
        return

    def __PixelIsosurfaceControl(self):
        regionNames = self.GetCheckedRegions()

        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            try:
                regionNames.index(regionName)
                self.__vtkPanel.SetPixelIsosurface(regionName, self.GetParameter(qspace.TAG_OBJ3D_SURFSCALAR), self.GetParameter(qspace.TAG_OBJ3D_PXNO))
                self.__vtkPanel.ShowPixelIsosurface(regionName, self.__pixelIsosurfaceControlMenuItem.IsChecked())
                pass
            except ValueError:
                self.__vtkPanel.ShowPixelIsosurface(regionName, False)
                pass
            pass
        return

    def __LatticeAxesControl(self):
        self.__vtkPanel.ShowLatticeAxes(self.__latticeAxesControlMenuItem.IsChecked())
        return

    def __UpdateRendering(self):
        self.__RegionSurfaceControl()
        self.__EtIsosurfaceControl()
        self.__vtkPanel.ShowQPoint(self.__qPointDisplayMenuItem.IsChecked())
        self.__ElasticScatteringLineControl()
        self.__CellDisplayControl()
        self.__RotationGuideControl()
        self.__PixelIsosurfaceControl()
        self.__LatticeAxesControl()
        return

    def ShowPSD(self, event):
        regionName = self.__regionNameMap[event.GetId()]
        menuItem = self.__displayControlMenu.FindItemById(event.GetId())
        psdNumber = int(menuItem.GetLabel())

        self.__vtkPanel.ShowPSDOutline(regionName, psdNumber, menuItem.IsChecked())
        return

    def __createRegionMenu(self, menuBar):
        regionMenu = wx.Menu()

        self.__regionDataSubMenu = wx.Menu()
        regionMenu.AppendSubMenu(self.__regionDataSubMenu, 'Data View')

        self.__selectAllRegionMenuItem = self.__regionDataSubMenu.Append(wx.ID_ANY, 'Select all').GetId()
        self.Bind(wx.EVT_MENU, self.AllRegionControl, id = self.__selectAllRegionMenuItem)

        self.__resetAllRegionMenuItem = self.__regionDataSubMenu.Append(wx.ID_ANY, 'Reset all').GetId()
        self.Bind(wx.EVT_MENU, self.AllRegionControl, id = self.__resetAllRegionMenuItem)

        self.__regionDataSubMenu.AppendSeparator()

        for name in self.__qSpaceManager.param.GetAllRegionNames():
            menuItem = self.__regionDataSubMenu.AppendCheckItem(wx.ID_ANY, name)
            menuItem.Check(True)
            self.Bind(wx.EVT_MENU, self.SelectRegionMenuItem, menuItem)
            self.Bind(wx.EVT_MENU_HIGHLIGHT, self.HighlightRegionMenuItem, menuItem)
            pass

        self.__regionFrameSubMenu = wx.Menu()
        regionMenu.AppendSubMenu(self.__regionFrameSubMenu, 'Frame View')

        self.__selectAllRegionFrameMenuItem = self.__regionFrameSubMenu.Append(wx.ID_ANY, 'Select all').GetId()
        self.Bind(wx.EVT_MENU, self.AllRegionFrameControl, id = self.__selectAllRegionFrameMenuItem)

        self.__resetAllRegionFrameMenuItem = self.__regionFrameSubMenu.Append(wx.ID_ANY, 'Reset all').GetId()
        self.Bind(wx.EVT_MENU, self.AllRegionFrameControl, id = self.__resetAllRegionFrameMenuItem)

        self.__regionFrameSubMenu.AppendSeparator()

        for name in self.__qSpaceManager.param.GetAllRegionNames():
            menuItem = self.__regionFrameSubMenu.AppendCheckItem(wx.ID_ANY, name)
            self.Bind(wx.EVT_MENU, self.SelectRegionFrameMenuItem, menuItem)
            self.Bind(wx.EVT_MENU_HIGHLIGHT, self.HighlightRegionMenuItem, menuItem)
            pass

        self.Bind(wx.EVT_MENU_CLOSE, self.RegionMenuClose)

        menuBar.Append(regionMenu, 'Region')
        return

    def AllRegionControl(self, event):
        for menuItem in self.__regionDataSubMenu.GetMenuItems():
            if menuItem.IsCheckable():
                if event.GetId() == self.__selectAllRegionMenuItem:
                    menuItem.Check(True)
                    pass
                else:
                    menuItem.Check(False)
                    pass
                pass
            pass

        if event.GetId() == self.__selectAllRegionMenuItem:
            self.__qSpaceManager.UpdateBoundingBox(self.GetCheckedRegions())
            bounds = self.GetBounds()
            self.__vtkPanel.SetAxesRange(bounds)
            pass

        self.__UpdateRegionDisplay()
        return

    def HighlightRegionMenuItem(self, event):
        item = self.__regionDataSubMenu.FindItemById(event.GetId())
        if item == None:
            item = self.__regionFrameSubMenu.FindItemById(event.GetId())
            pass

        regionName = item.GetLabel()

        self.__vtkPanel.ShowDetectorRegionHighlight(regionName)

        self.__vtkPanel.Render()
        return

    def __UpdateRegionDisplay(self):
        self.__qSpaceManager.UpdateBoundingBox(self.GetCheckedRegions())
        bounds = self.GetBounds()
        self.__vtkPanel.SetAxesRange(bounds)

        if self.__regionSurfaceControlMenuItem.IsChecked():
            self.__RegionSurfaceControl()
            pass

        if self.__etIsosurfaceControlMenuItem.IsChecked():
            self.__EtIsosurfaceControl()
            pass

        if self.__elasticScatteringLineMenuItem.IsChecked():
            self.__ElasticScatteringLineControl()
            pass

        if self.__pixelIsosurfaceControlMenuItem.IsChecked():
            self.__PixelIsosurfaceControl()
            pass

        for menuItem in self.__regionFrameSubMenu.GetMenuItems():
            if menuItem.IsCheckable():
                self.__vtkPanel.ShowDetectorRegion(menuItem.GetLabel(), menuItem.IsChecked())
                pass
            pass

        self.__vtkPanel.Render()
        return

    def SelectRegionMenuItem(self, event):
        menuItem = self.__regionDataSubMenu.FindItemById(event.GetId())

        self.__UpdateRegionDisplay()
        return

    def RegionMenuClose(self, event):
        self.__vtkPanel.HideDetectorRegionHighlight()
        self.__vtkPanel.Render()
        return

    def AllRegionFrameControl(self, event):
        for menuItem in self.__regionFrameSubMenu.GetMenuItems():
            if menuItem.IsCheckable():
                if event.GetId() == self.__selectAllRegionFrameMenuItem:
                    menuItem.Check(True)
                    pass
                else:
                    menuItem.Check(False)
                    pass

                self.__vtkPanel.ShowDetectorRegion(menuItem.GetLabel(), menuItem.IsChecked())
                pass
            pass

        self.__vtkPanel.Render()
        return

    def SelectRegionFrameMenuItem(self, event):
        menuItem = self.__regionFrameSubMenu.FindItemById(event.GetId())

        self.__vtkPanel.ShowDetectorRegion(menuItem.GetLabel(), menuItem.IsChecked())

        self.__vtkPanel.Render()
        return

    def __createControlMenu(self, menuBar):
        controlMenu = wx.Menu()

        menuItem = controlMenu.Append(wx.ID_ANY, 'Rotation control...\tCtrl-R')
        self.Bind(wx.EVT_MENU, self.PopupRotationControlDialog, menuItem)
        menuItem = controlMenu.Append(wx.ID_ANY, 'Energy control...\tCtrl-E')
        self.Bind(wx.EVT_MENU, self.PopupEnergyControlDialog, menuItem)
        menuItem = controlMenu.Append(wx.ID_ANY, 'Scalar Range Settings...')
        self.Bind(wx.EVT_MENU, self.PopupScalarRangeDialog, menuItem)

        menuBar.Append(controlMenu, 'Control')
        return

    def __createGraphMenu(self, menuBar):
        graphMenu = wx.Menu()
        menuBar.Append(graphMenu, 'Graph')

        menuItem = graphMenu.Append(wx.ID_ANY, 'QE Graph...\tCtrl-Q')
        self.Bind(wx.EVT_MENU, self.PopupQEGraphSetupDialog, menuItem)
        menuItem = graphMenu.Append(wx.ID_ANY, 'QRegion Graph...')
        self.Bind(wx.EVT_MENU, self.PopupQRegionSetupDialog, menuItem)
        return

    def __createToolBar(self):
        def AddTool(toolBar, pngPath, shortHelp, longHelp):
            import os

            image = wx.Image(os.path.join(os.path.abspath(os.path.dirname(__file__)), pngPath), wx.BITMAP_TYPE_PNG)
            bitmap = image.ConvertToBitmap()
            return toolBar.AddSimpleTool(wx.ID_ANY, bitmap, shortHelp, longHelp)

        self.__toolBar = wx.ToolBar(self)
        self.SetToolBar(self.__toolBar)

        toolItem = AddTool(self.__toolBar, 'logos/tool_s.png',
                           'Sample Information settings...', 'Open Sample Information settings dialog')
        self.Bind(wx.EVT_TOOL, self.PopupSampleInformationDialog, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_ei.png',
                           'Incident energy control...', 'Open incident energy control dialog')
        self.Bind(wx.EVT_TOOL, self.PopupEnergyControlDialog, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_qe.png',
                           'QE graph settings...', 'Open QE graph settings dialog')
        self.Bind(wx.EVT_TOOL, self.PopupQEGraphSetupDialog, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_rc.png',
                           'Rotation control...', 'Open rotation control dialog')
        self.Bind(wx.EVT_TOOL, self.PopupRotationControlDialog, toolItem)

        self.__toolBar.AddSeparator()

        toolItem = AddTool(self.__toolBar, 'logos/tool_qr.png',
                           'QR graph settings...', 'Open QR graph settings dialog')
        self.Bind(wx.EVT_TOOL, self.PopupQRegionSetupDialog, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_obj.png',
                           '3D object settings...', 'Open 3D Object settings dialog')
        self.Bind(wx.EVT_TOOL, self.PopupObjectSettingDialog, toolItem)

        self.__toolBar.AddSeparator()

        toolItem = AddTool(self.__toolBar, 'logos/tool_x.png',
                           'qx view', 'Change view along X axis')
        self.Bind(wx.EVT_TOOL, self.ViewControl, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_y.png',
                           'qy view', 'Change view along Y axis')
        self.Bind(wx.EVT_TOOL, self.ViewControl, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_z.png',
                           'qz view', 'Change view along Z axis')
        self.Bind(wx.EVT_TOOL, self.ViewControl, toolItem)

        self.__toolBar.AddSeparator()

        toolItem = AddTool(self.__toolBar, 'logos/tool_a.png',
                           'qa view', 'Change view along a axis of lattice')
        self.Bind(wx.EVT_TOOL, self.ViewControl, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_b.png',
                           'qb view', 'Change view along b axis of lattice')
        self.Bind(wx.EVT_TOOL, self.ViewControl, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_c.png',
                           'qc view', 'Change view along c axis of lattice')
        self.Bind(wx.EVT_TOOL, self.ViewControl, toolItem)

        self.__toolBar.AddSeparator()

        toolItem = AddTool(self.__toolBar, 'logos/tool_pev.png',
                           'Perspective view', 'Change perspective view mode')
        self.Bind(wx.EVT_TOOL, self.ProjectionControl, toolItem)
        toolItem = AddTool(self.__toolBar, 'logos/tool_ppv.png',
                           'Parallel-projection view', 'Change parallel-projection view mode')
        self.Bind(wx.EVT_TOOL, self.ProjectionControl, toolItem)
        self.__toolBar.Realize()
        return

    def __updateQPointList(self):
        listControl = self.getControl('QpListCtrl')
        listControl.ClearAll()
        listControl.InsertColumn(0, 'Point ID')
        listControl.InsertColumn(1, 'Et [meV]')
        listControl.SetColumnWidth(0, 100)
        listControl.SetColumnWidth(1, 100)

        colors = wx.ColourDatabase()
        pink = colors.Find("PINK")

        self.__qSpaceManager.CreateDetectableRegion(self.__mode)
        self.__qSpaceManager.UpdateBoundingBox(self.GetCheckedRegions())
        self.__qSpaceManager.UpdateQpPositions(self.__mode)
        qPointList = self.__qSpaceManager.GetVisibleQPoints()

        for p in qPointList:
            key = p.GetIndexString()
            index = listControl.InsertStringItem(0, key)

            if p.GetEtmax() == None:
                text = 'None'
                pass
            else:
                text = "% g" % p.GetEtmax()
                pass
            listControl.SetStringItem(index, 1, text)

            selectKey = self.GetParameter(qspace.TAG_SELECTQPINDEX)
            if selectKey != None and selectKey == key:
                listControl.SetItemState(index, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
                pass

            if p.numInsidePoints > 0:
                listControl.SetItemBackgroundColour(index, pink)
                pass
            pass
        return

    def GetScalarFieldRange(self, type):
        return self.__qSpaceManager.GetScalarFieldRange(type, self.__isMeasurementDataLoaded)

    def UpdateView(self):
        scalarKind = self.GetScalarKind()

        scalarRange = self.GetScalarFieldRange(scalarKind)

        self.__vtkPanel.CreateColorBar(scalarKind, scalarRange)

        drawingData = self.__qSpaceManager.GetDetectorRegions()
        self.__vtkPanel.CreateDetectorRegionView(drawingData, scalarRange,
                                                 self.__qSpaceManager.param.detectorProps)

        if self.GetScalarKind() == define.DETECT_INTENSITY:
            drawingData = self.__qSpaceManager.GetIntensityRegions()
            self.__vtkPanel.CreateIntensityRegionView(drawingData, scalarRange, self.__qSpaceManager.param.Eng_array)
            pass

        bounds = self.GetBounds()
        radius = max(bounds[1] - bounds[0],
                     bounds[3] - bounds[2],
                     bounds[5] - bounds[4]) * self.__qPointSize

        qPointList = self.__qSpaceManager.GetVisibleQPoints()
        self.__vtkPanel.CreateQPoints(qPointList, radius)

        self.__vtkPanel.SetAxesRange(bounds)
        self.__vtkPanel.CreateCameraView('qy', self.__qSpaceManager.GetBoundingSize(), self.__qSpaceManager.GetCenter(), bounds[1], bounds[3], bounds[5])

        self.__vtkPanel.CreateEtIsosurface()
        self.__vtkPanel.CreatePixelIsosurface()

        self.__vtkPanel.CreateElasticScatteringLines()
        self.__vtkPanel.CreateCell(self.__qSpaceManager.ltc.ra_rotate,
                                   self.__qSpaceManager.ltc.rb_rotate,
                                   self.__qSpaceManager.ltc.rc_rotate)
        self.__vtkPanel.CreateRotationGuide()
        self.__vtkPanel.CreateLatticeAxes((self.__qSpaceManager.ltc.ra_rotate,
                                           self.__qSpaceManager.ltc.rb_rotate,
                                           self.__qSpaceManager.ltc.rc_rotate),
                                          self.__qSpaceManager.GetDiagonalBoundingSize())
        return

    def GetCheckedRegions(self):
        regions = []

        for menuItem in self.__regionDataSubMenu.GetMenuItems():
            if menuItem.IsCheckable() and menuItem.IsChecked():
                regions.append(menuItem.GetItemLabel())
                pass
            pass
        return regions

    def MethodFileAccess(self, event):
        itemName = self.__fileMenu.FindItemById(event.GetId()).GetLabel()

        if itemName == 'Load method file':
            title = 'Select read file'
            dialogStyle = wx.FD_OPEN | wx.FD_FILE_MUST_EXIST
            pass
        else:
            title = 'Save file'
            dialogStyle = wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
            pass

        dlg = wx.FileDialog(self,
                            title, defaultFile = 'Default.amt', wildcard = 'AMT Files (*.amt)|*.amt',
                            style = dialogStyle)

        if dlg.ShowModal() == wx.ID_OK:
            filepath = dlg.GetPath()

            if itemName == 'Load method file':
                self.__qSpaceManager.status.Read(filepath)

                self.__rotationControlDialog.ReloadParameters()
                pass
            else:
                self.__qSpaceManager.status.OutputStatus(filepath)
                pass
            pass
        return

    def __energyRangeCheck(self, energyRange):
        if energyRange[0] != self.GetFloatParameter(qspace.TAG_ENG_EI) or \
                energyRange[1] != self.GetFloatParameter(qspace.TAG_ENG_ETMIN) or \
                energyRange[2] != self.GetFloatParameter(qspace.TAG_ENG_ETMAX):
            message = 'Do you adjust current energy settings to following range of BINs in NeXus file ?\n' \
                + '  Ei, Etmin, Etmax: %g, %g, %g [meV]' % energyRange

            dlg = wx.MessageDialog(self, message, 'Question for energy settings',
                                   wx.YES_NO | wx.ICON_QUESTION | wx.STAY_ON_TOP)

            if dlg.ShowModal() == wx.ID_YES:
                import decimal

                # if self.__mode == 'DNA':
                #     self.SetParameter(qspace.TAG_ENG_EIMIN, decimal.Decimal(str(energyRange[1])))
                #     self.SetParameter(qspace.TAG_ENG_EIMAX, decimal.Decimal(str(energyRange[2])))
                #     pass
                # else:
                #     self.SetParameter(qspace.TAG_ENG_EI, decimal.Decimal(str(energyRange[0])))
                #     self.SetParameter(qspace.TAG_ENG_ETMIN, decimal.Decimal(str(energyRange[1])))
                #     self.SetParameter(qspace.TAG_ENG_ETMAX, decimal.Decimal(str(energyRange[2])))
                #     pass

                self.SetParameter(qspace.TAG_ENG_EI, decimal.Decimal(str(energyRange[0])))
                self.SetParameter(qspace.TAG_ENG_ETMIN, decimal.Decimal(str(energyRange[1])))
                self.SetParameter(qspace.TAG_ENG_ETMAX, decimal.Decimal(str(energyRange[2])))

                self.__qSpaceManager.UpdateEnergyRange()
                pass
            pass
        return

    def LoadDataFile(self, event):
        itemName = self.__fileMenu.FindItemById(event.GetId()).GetLabel()

        if itemName == 'Load Boost file':
            title = 'Load Boost file'
            wildcard = 'Boost files (*.bst)|*.bst'
            self.__isHDF = False
            pass
        else:
            title = 'Load Boost file'
            wildcard = 'NeXus files (*.nx)|*.nx'
            self.__isHDF = True
            pass

        dlg = wx.FileDialog(self,
                            title, wildcard = wildcard,
                            style = wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)

        if dlg.ShowModal() == wx.ID_OK:
            self.__filepath = dlg.GetPath()

            self.__workerMode = 'file loading'

            self.__progressBarDialog = dialog.ProgressBarDialog(self, 'Please wait for loading file', 0)
            self.__progressBarDialog.Show()

            EVT_WATCHER_UPDATE(self.__progressBarDialog, self.__progressBarDialog.Update)

            WorkerWatcher(self.FileLoading, self.__progressBarDialog, self).start()

            self.__isMeasurementDataLoaded = True
            pass
        return

    def FileLoading(self):
        self.__energyRange = self.__qSpaceManager.LoadDataFromFile(self.__filepath, self.__isHDF)
        return

    def DataMapping(self):
        self.__qSpaceManager.MeasurementDataMapping(self.__mode)

        # self.__updateQPointList()
        # self.UpdateView()
        # self.__UpdateRendering()
        print 'end'
        return

    def WorkerFinished(self, event):
        self.__progressBarDialog.Show(False)

        if self.__workerMode == 'file loading':
            self.__workerMode = 'data mapping'

            self.__energyRangeCheck(self.__energyRange)

            self.__progressBarDialog = dialog.ProgressBarDialog(self, 'Please wait for data mapping', 0)
            self.__progressBarDialog.Show()

            EVT_WATCHER_UPDATE(self.__progressBarDialog, self.__progressBarDialog.Update)

            WorkerWatcher(self.DataMapping, self.__progressBarDialog, self).start()
            pass
        elif self.__workerMode == 'data mapping':
            print 'update'
            self.__updateQPointList()
            self.UpdateView()
            self.__UpdateRendering()
            pass
        elif self.__workerMode == 'update view':
            self.__UpdateRendering()
            pass
        return

    def close(self, event):
        self.Close()
        return

    def lstColumnClick(self, event):
        if self.__lstSortMode == 'IJK':
            self.__lstSortMode = 'ET'
            pass
        else:
            self.__lstSortMode = 'IJK'
            pass

        self.__updateQPointList()
        self.ApplySettings()
        return

    def __SelectQPoint(self, qPointIndex):
        self.__vtkPanel.SelectedQPoint(qPointIndex, self.__qSpaceManager.GetQPoint(qPointIndex))
        self.SetParameter(qspace.TAG_SELECTQPINDEX, qPointIndex)

        if self.__rotationGuideControlMenuItem.IsChecked():
            self.__RotationGuideControl()
            pass

        self.__vtkPanel.Render()
        return

    def QPointListSetSelected(self, qPointIndex):
        listControl = self.getControl('QpListCtrl')

        for i in xrange(listControl.GetItemCount()):
            if listControl.GetItemText(i) == qPointIndex:
                listControl.SetItemState(i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
                break
            pass

        self.__PopupQPointInformationDialog(listControl.GetItemText(i))
        return

    def lstSelected(self, event):
        self.__SelectQPoint(event.GetItem().GetText())
        return

    def lstDeselected(self, event):
        key = event.GetItem().GetText()
        self.__vtkPanel.DeselectedQPoint(key, self.__qPointDisplayMenuItem.IsChecked())
        return

    def __PopupQPointInformationDialog(self, qPointIndex):
        qPoint = self.__qSpaceManager.GetQPoint(qPointIndex)

        dlgQpInfo = dialog.QPointInformationDialog(self, qPoint.GetInfoText())
        dlgQpInfo.ShowModal()
        dlgQpInfo.Destroy()
        return

    def lstActivated(self, event):
        listControl = self.getControl('QpListCtrl')
        item = listControl.GetFirstSelected()

        if item == -1:
            return

        self.__PopupQPointInformationDialog(listControl.GetItemText(item))
        return

    def lstRightClicked(self, event):
        # Test Script for BeamIntensity module
        import BeamIntensity

        mod = BeamIntensity.Moderator()
        mod.ReadIntensityData('Pulse_0310_mod_port02.csv')

        bl = BeamIntensity.BeamLine(mod)

        ch = BeamIntensity.DoubleDiskChopper("Pulse shaping chopper (No.1)", 7.75, 0.05, 300.0, 0.0246, 0.090, 0.3)
        ch.thisown = 0

        ch.AddSlitToForwardChopper(5.7, 0.0)
        ch.AddSlitToForwardChopper(5.7, 90.0)
        ch.AddSlitToForwardChopper(5.7, 180.0)
        ch.AddSlitToForwardChopper(1.9, 270.0)

        ch.AddSlitToBackwardChopper(5.7, 0.0)
        ch.AddSlitToBackwardChopper(5.7, -90.0)
        ch.AddSlitToBackwardChopper(5.7, -180.0)
        ch.AddSlitToBackwardChopper(1.9, -270.0)

        ch.SetInitialPhase(-74.40024511, 74.40024511) # for 2nd frame

        bl.AddChopper(ch)

        ch = BeamIntensity.DoubleDiskChopper('Slow double disk chopper (No.4)', 10.0, 0.04, 25.0, 0.0414, 0.090, 0.3)
        ch.thisown = 0
        ch.AddSlitToForwardChopper(85.7, 0.0)
        ch.AddSlitToBackwardChopper(85.7, 0.0)
        ch.SetInitialPhase(-123.8677683, 123.8677683) # for 2nd frame

        bl.AddChopper(ch)

        ch = BeamIntensity.SingleDiskChopper('Frame separation chopper (No.2)', 11.625, 200.0, 0.0492, 0.090, 0.3)
        ch.thisown = 0

        ch.AddSlit(24.7, 0.0)
        ch.AddSlit(24.7, 90.0)
        ch.AddSlit(24.7, 180.0)
        ch.AddSlit(24.7, 270.0)
        ch.SetInitialPhase(-70.80024511) # for 2nd frame

        bl.AddChopper(ch)

        ch = BeamIntensity.SingleDiskChopper('Slow single disk chopper (No.5)', 12.5, 25.0, 0.0525, 0.090, 0.3)
        ch.thisown = 0
        ch.AddSlit(118.9, 0.0)
        ch.SetInitialPhase(-154.6097104) # for 2nd frame

        bl.AddChopper(ch)

        ch = BeamIntensity.DoubleDiskChopper('Slow double disk chopper (No.6)', 15.0, 0.04, 25.0, 0.060, 0.090, 0.3)
        ch.thisown = 0
        ch.AddSlitToForwardChopper(128.6, 0.0)
        ch.AddSlitToBackwardChopper(128.6, 0.0)
        ch.SetInitialPhase(-185.3516524, 185.3516524) # for 2nd frame

        bl.AddChopper(ch)

        ch = BeamIntensity.SingleDiskChopper('Frame separation chopper (No.3)', 23.25, 200.0, 0.06, 0.090, 0.3)
        ch.thisown = 0
        ch.AddSlit(56.6, 0.0)
        ch.AddSlit(56.6, 180.0)
        ch.SetInitialPhase(-134.4004902) # for 2nd frame

        bl.AddChopper(ch)

        calculator = BeamIntensity.IntensityCalculator(1.0e-5, # [s]
                                                       90.0e-3, #[s]
                                                       8.0, # [meV]
                                                       1.0e-3, # [s]
                                                       46.3, # [m]
                                                       bl)
        calculator.CalculateIntensity()

        itval = numpy.zeros((2, calculator.GetNumberOfTimes()))
        ieval = numpy.zeros((2, calculator.GetNumberOfTimes()))

        for i in xrange(0, calculator.GetNumberOfTimes()):
            itval[0][i] = calculator.GetTimePoint(i)
            itval[1][i] = calculator.GetIntensityByTime(i)
            ieval[0][i] = calculator.GetEnergyPoint(i)
            ieval[1][i] = calculator.GetIntensityByEnergy(i)
            pass

        import matplotlib.pyplot as plt

        plt.subplot(211)
        plt.xlabel('Time of flight [s]')
        plt.ylabel('Energy integrated intensity\n[n/cm2/s/sr/pulse]')

        plt.xlim(itval[0][0], itval[0][calculator.GetNumberOfTimes() - 1])
        plt.plot(itval[0], itval[1])
        plt.grid()

        plt.subplot(212)
        plt.xlabel('Neutron energy [meV]')
        plt.ylabel('Time integrated intensity\n[n/cm2/sr/eV/pulse]')

        plt.xlim(ieval[0][calculator.GetNumberOfTimes() - 1], ieval[0][0])
        plt.plot(ieval[0], ieval[1])
        plt.grid()

        plt.show()
        plt.clf()

        del bl, mod, calculator
        return

    def PopupRotationControlDialog(self, event):
        listControl = self.getControl('QpListCtrl')
        item = listControl.GetFirstSelected()

        if item == -1:
            self.__rotationControlDialog.SetQPointInformation('No point selected.')
            pass
        else:
            qPoint = self.__qSpaceManager.GetQPoint(listControl.GetItemText(item))
            self.__rotationControlDialog.SetQPointInformation(qPoint.GetInfoText())
            pass

        self.__rotationControlDialog.Show()
        return

    def UpdateRotation(self):
        listControl = self.getControl('QpListCtrl')
        item = listControl.GetFirstSelected()

        import math

        psi = math.radians(self.GetParameter(qspace.TAG_ROTATE_PSI))
        chi = math.radians(self.GetParameter(qspace.TAG_ROTATE_CHI))
        phi = math.radians(self.GetParameter(qspace.TAG_ROTATE_PHI))

        self.__qSpaceManager.ltc.SetRotation(psi, chi, phi)

        self.__RotationGuideControl()

        self.__qSpaceManager.UpdateQpPositions(self.__mode)

        bounds = self.GetBounds()
        radius = max(bounds[1] - bounds[0],
                     bounds[3] - bounds[2],
                     bounds[5] - bounds[4]) * self.__qPointSize

        qPointList = self.__qSpaceManager.GetVisibleQPoints()
        self.__vtkPanel.CreateQPoints(qPointList, radius)

        self.__vtkPanel.ShowQPoint(self.__qPointDisplayMenuItem.IsChecked())

        self.__vtkPanel.UpdateCell(self.__qSpaceManager.ltc.ra_rotate,
                                   self.__qSpaceManager.ltc.rb_rotate,
                                   self.__qSpaceManager.ltc.rc_rotate)

        self.__vtkPanel.UpdateLatticeAxes((self.__qSpaceManager.ltc.ra_rotate,
                                           self.__qSpaceManager.ltc.rb_rotate,
                                           self.__qSpaceManager.ltc.rc_rotate),
                                          self.__qSpaceManager.GetDiagonalBoundingSize())

        qPointKey = listControl.GetItemText(item)

        if qPointKey != None and qPointKey != '':
            selectedQPoint = self.__qSpaceManager.GetQPoint(qPointKey)

            self.__rotationControlDialog.SetQPointInformation(selectedQPoint.GetInfoText())

            self.__vtkPanel.SelectedQPoint(qPointKey, selectedQPoint)
            pass

        self.__vtkPanel.Render()
        return

    def UpdateRotationGuideAxis(self):
        if self.__rotationGuideControlMenuItem.IsChecked():
            self.__RotationGuideControl()
            self.__vtkPanel.Render()
            pass
        return

    def UpdateSampleInformation(self):
        self.__qSpaceManager.ltc = qspace.Lattice(self.__qSpaceManager.status)

        self.UpdateRotation()
        return

    def PopupEnergyControlDialog(self, event):
        if self.__mode == 'DNA':
            dlg = dialog.DNAEnergyControlDialog(self)
            pass
        else:
            dlg = dialog.SIKEnergyControlDialog(self)
            pass

        if dlg.ShowModal() == wx.ID_OK:
            if self.__mode == 'DNA':
                range = dlg.GetEnergyRange()
                self.SetParameter(qspace.TAG_ENG_EIMIN, range[0])
                self.SetParameter(qspace.TAG_ENG_EIMAX, range[1])
                pass
            else:
                range = dlg.GetEnergyRange()
                self.SetParameter(qspace.TAG_ENG_EI, range[0])
                self.SetParameter(qspace.TAG_ENG_ETMIN, range[1])
                self.SetParameter(qspace.TAG_ENG_ETMAX, range[2])
                pass

            listControl = self.getControl('QpListCtrl')
            item = listControl.GetFirstSelected()

            if item != -1:
                qPointKey = listControl.GetItemText(item)
                pass
            else:
                qPointKey = None
                pass

            self.__qSpaceManager.UpdateEnergyRange()
            self.__updateQPointList()
            self.ApplySettings()

            if qPointKey != None:
                self.__vtkPanel.SelectedQPoint(qPointKey, self.__qSpaceManager.GetQPoint(qPointKey))
                pass

            self.__vtkPanel.Render()
            pass
        return

    def PopupScalarRangeDialog(self, event):
        dlg = dialog.ScalarRangeDialog(self)

        if dlg.ShowModal() == wx.ID_OK:
            dlg.UpdateSettings()
            pass
        return

    def PopupQEGraphSetupDialog(self, event):
        self.ShowQEGraphCutPlane(True)

        dlg = dialog.QEGraphSetupDialog(self)
        dlg.Show()
        return

    def GetQEGraphAxis(self):
        planeType = self.GetParameter(qspace.TAG_QEGRAPH_PLANEDEF)

        oid = [self.GetParameter(qspace.TAG_QEGRAPH_OH),
               self.GetParameter(qspace.TAG_QEGRAPH_OK),
               self.GetParameter(qspace.TAG_QEGRAPH_OL)]

        qO = self.__qSpaceManager.GetRotateQPosition(oid)

        if planeType == 0:
            qU = self.__qSpaceManager.ltc.ra_rotate
            qV = self.__qSpaceManager.ltc.rb_rotate
            pass
        elif planeType == 1:
            qU = self.__qSpaceManager.ltc.rb_rotate
            qV = self.__qSpaceManager.ltc.rc_rotate
            pass
        elif planeType == 2:
            qU = self.__qSpaceManager.ltc.rc_rotate
            qV = self.__qSpaceManager.ltc.ra_rotate
            pass
        else:
            uid = [self.GetParameter(qspace.TAG_QEGRAPH_UH),
                   self.GetParameter(qspace.TAG_QEGRAPH_UK),
                   self.GetParameter(qspace.TAG_QEGRAPH_UL)]

            vid = [self.GetParameter(qspace.TAG_QEGRAPH_VH),
                   self.GetParameter(qspace.TAG_QEGRAPH_VK),
                   self.GetParameter(qspace.TAG_QEGRAPH_VL)]

            qU = self.__qSpaceManager.GetRotateQPosition(uid)
            qV = self.__qSpaceManager.GetRotateQPosition(vid)
            pass

        qNormal = vector.GetCross(qU, qV)
        return qO, qU, qV, qNormal

    def UpdateQEGraphCutPlane(self):
        qOrigin, qU, qV, qNormal = self.GetQEGraphAxis()

        bounds = self.GetBounds()
        self.__vtkPanel.UpdateQEGraphPlane(bounds, qOrigin, qNormal)

        # if vector.GetNorm(qNormal) != 0.0:
        #     self.__vtkPanel.ShowQEGraphPlane(True)
        #     pass
        # else:
        #     self.__vtkPanel.ShowQEGraphPlane(False)
        #     pass

        self.__vtkPanel.Render()
        return

    def ShowQEGraphCutPlane(self, isShow):
        self.__vtkPanel.ShowQEGraphPlane(isShow)

        regionNames = self.GetCheckedRegions()

        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            try:
                regionNames.index(regionName)
                self.__vtkPanel.ShowQEGraphRegionPlane(regionName, isShow)
                pass
            except ValueError:
                self.__vtkPanel.ShowQEGraphRegionPlane(regionName, False)
                pass
            pass

        self.__vtkPanel.Render()
        return

    def PopupQRegionSetupDialog(self, event):
        regionPoints = self.GetAllThinDetectableRegions()
        self.__vtkPanel.CreateThinDetectableRegionShape(regionPoints)

        regionNames = self.GetCheckedRegions()

        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            try:
                regionNames.index(regionName)
                self.__vtkPanel.ShowThinDetectableRegionShape(regionName, True)
                pass
            except ValueError:
                self.__vtkPanel.ShowThinDetectableRegionShape(regionName, False)
                pass
            pass

        self.ShowQRigionGraphCutPlane(True)

        dlg = dialog.QRegionSetupDialog(self)
        dlg.Show()
        return

    def UpdateQRegionGraphCutPlane(self):
        bounds = self.GetBounds()

        qO0, qU0, qV0 = self.GetQRegionGraphAxis('start')
        qO1, qU1, qV1 = self.GetQRegionGraphAxis('end')

        qNormal0 = vector.GetCross(qU0, qV0)
        qNormal1 = vector.GetCross(qU1, qV1)

        self.__vtkPanel.UpdateQRegionGraphPlane(bounds, (qO0, qO1), (qNormal0, qNormal1), self.GetCheckedRegions())

        self.__vtkPanel.Render()
        return

    def ShowQRigionGraphCutPlane(self, isShow):
        self.__vtkPanel.ShowQRegionGraphPlane(isShow)

        regionNames = self.GetCheckedRegions()

        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            try:
                regionNames.index(regionName)
                self.__vtkPanel.ShowQRegionGraphCutRegion(regionName, isShow)
                pass
            except ValueError:
                self.__vtkPanel.ShowQRegionGraphCutRegion(regionName, False)
                pass
            pass

        self.__vtkPanel.Render()
        return

    def GetQRegionGraphAxis(self, rotatePosition = None):
        import math

        ra = self.__qSpaceManager.ltc.ra_orig
        rb = self.__qSpaceManager.ltc.rb_orig
        rc = self.__qSpaceManager.ltc.rc_orig

        qO = vector.GetVectorByIndex(ra, rb, rc,
                                     self.GetParameter(qspace.TAG_QRGRAPH_OH),
                                     self.GetParameter(qspace.TAG_QRGRAPH_OK),
                                     self.GetParameter(qspace.TAG_QRGRAPH_OL))

        planeType = self.GetParameter(qspace.TAG_QRGRAPH_PLANEDEF)

        if planeType == 0:
            qU = ra
            qV = rb
            pass
        elif planeType == 1:
            qU = rb
            qV = rc
            pass
        elif planeType == 2:
            qU = rc
            qV = ra
            pass
        else:
            qU = vector.GetVectorByIndex(ra, rb, rc,
                                         self.GetParameter(qspace.TAG_QRGRAPH_UH),
                                         self.GetParameter(qspace.TAG_QRGRAPH_UK),
                                         self.GetParameter(qspace.TAG_QRGRAPH_UL))

            qV = vector.GetVectorByIndex(ra, rb, rc,
                                         self.GetParameter(qspace.TAG_QRGRAPH_VH),
                                         self.GetParameter(qspace.TAG_QRGRAPH_VK),
                                         self.GetParameter(qspace.TAG_QRGRAPH_VL))
            pass

        psi = math.radians(self.GetParameter(qspace.TAG_QRGRAPH_PSI0))
        chi = math.radians(self.GetParameter(qspace.TAG_QRGRAPH_CHI0))
        phi = math.radians(self.GetParameter(qspace.TAG_QRGRAPH_PHI0))

        if rotatePosition == 'start':
            M = VTKPanel.GetRotationMatrix(psi, chi, phi)

            qO = qspace.ApplyRotMat(M, qO)
            qU = qspace.ApplyRotMat(M, qU)
            qV = qspace.ApplyRotMat(M, qV)
            pass
        elif rotatePosition == 'end':
            rotateAxis = self.GetParameter(qspace.TAG_QRGRAPH_ROTATEAXIS)
            dTheta = math.radians(self.GetParameter(qspace.TAG_QRGRAPH_INTERVAL)) * self.GetParameter(qspace.TAG_QRGRAPH_NSTEP)

            if rotateAxis == 0:
                psi += dTheta
                pass
            elif rotateAxis == 1:
                chi += dTheta
                pass
            elif rotateAxis == 2:
                phi += dTheta
                pass
            else:
                print 'Rotation axis error is found!'
                pass

            M = VTKPanel.GetRotationMatrix(psi, chi, phi)

            qO = qspace.ApplyRotMat(M, qO)
            qU = qspace.ApplyRotMat(M, qU)
            qV = qspace.ApplyRotMat(M, qV)
            pass
        return qO, qU, qV

    def SetQPointColor(self):
        qPointList = self.__qSpaceManager.GetVisibleQPoints()

        self.__vtkPanel.SetQPointColor(qPointList)
        self.__vtkPanel.Render()
        return

    def SetQPointOpacity(self):
        qPointList = self.__qSpaceManager.GetVisibleQPoints()

        self.__vtkPanel.SetQPointOpacity(qPointList)
        self.__vtkPanel.Render()
        return

    def EraseThinDetectableRegions(self):
        for regionName in self.__qSpaceManager.param.GetAllRegionNames():
            self.__vtkPanel.ShowThinDetectableRegionShape(regionName, False)
            pass

        self.__vtkPanel.Render()
        return

    def GetFloatParameter(self, parameterName):
        return self.__qSpaceManager.status.GetFloat(parameterName)

    def GetParameter(self, parameterName):
        return self.__qSpaceManager.status.GetValue(parameterName)

    def SetParameter(self, parameterName, value):
        self.__qSpaceManager.status.SetValue(parameterName, value)
        return

    def SetQPointSize(self, size):
        self.__qPointSize = size

        bounds = self.GetBounds()
        radius = max(bounds[1] - bounds[0], \
                     bounds[3] - bounds[2], \
                     bounds[5] - bounds[4]) * self.__qPointSize

        qPointList = self.__qSpaceManager.GetVisibleQPoints()
        self.__vtkPanel.CreateQPoints(qPointList, radius)

        self.__vtkPanel.ShowQPoint(self.__qPointDisplayMenuItem.IsChecked())
        return

    def GetQPointSize(self):
        return self.__qPointSize

    def GetLattice(self):
        return self.__qSpaceManager.ltc

    def GetRegionData(self, dataType):
        regionData = dict()

        for regionName in self.GetCheckedRegions():
            if dataType == define.ENERGY_TRANSFER:
                regionData[regionName] = self.__vtkPanel.GetGridData(regionName)
                pass
            else:
                regionData[regionName] = self.__vtkPanel.GetMeasurementData(regionName)
                pass
            pass
        return regionData

    def GetScalarKind(self):
        return self.GetParameter(qspace.TAG_OBJ3D_SURFSCALAR)

    def GetQPPotionOnQECutPlane(self, normal, origin):
        return self.__vtkPanel.GetQPPotionOnQECutPlane(normal, origin)

    def GetAllThinDetectableRegions(self):
        etmin = self.__qSpaceManager.status.GetFloat(qspace.TAG_QRGRAPH_ETMIN)
        etmax = self.__qSpaceManager.status.GetFloat(qspace.TAG_QRGRAPH_ETMAX)
        regionPoints = self.__qSpaceManager.CreateThinDetectableRegion(self.__mode, etmin, etmax)
        return regionPoints

    def GetThinDetectableRegions(self, etmin, etmax):
        regionPoints = self.__qSpaceManager.CreateThinDetectableRegion(self.__mode, etmin, etmax)
        return regionPoints

if __name__ == "__main__":
    import sys

    if len(sys.argv) <= 1 or sys.argv[1] == 'DNA':
        mode = 'DNA'
        pass
    else:
        mode = 'SIK'
        pass

    app = wx.App(0)
    frame = MainFrame(mode)
    app.SetTopWindow(frame)
    frame.CenterOnScreen()
    frame.Show()
    app.MainLoop()
    pass



# -*- coding: utf-8 -*-

import sys
import string
import math

import Manyo.core
import ModuleTest as ModTest


eca = ModTest.eca
ecs = ModTest.ecs
ecm = ModTest.ecm
ecd = ModTest.ecd
e2  = ModTest.ec0 * 2.0

#ModTest.PrintElementContainer( eca )
#ModTest.PrintElementContainer( ecs )
#ModTest.PrintElementContainer( ecm )
#ModTest.PrintElementContainer( ecd )
#ModTest.PrintElementContainer( ModTest.ec0 * 2.0 )

# python ModTestArithmeticOperations.py > `date +"%Y%m%d"`.txt
# などとして、diffで差分を監視すれば良い？



import os
import sys
import argparse
import json

"""
Installation script to build Utsusemi on AlmaLinux 8, 9 and Ubuntu 22, 24

2025/7/2 1.0RC

$ ls src
LASTUPDATED  manyo/  uGao/  Utsusemi/  Utsusemi-DNA/  Utsusemi-ENG/  Utsusemi-SAS/
$ python3 Install.py
"""

DICT_BLCODE = {"1":"SIK", "2":"DNA", "10":"NBR", "11":"HPN", "14":"AMR", "15":"SAS", "17":"VNR", "19":"ENG"}

SOURCE_ALL_LIST = ["manyo", "Utsusemi", "Utsusemi-SAS", "Utsusemi-DNA", "Utsusemi-ENG"]
ALL_BL_LIST = ["SIK", "AMR", "HPN", "SAS", "DNA", "ENG", "VNR"]
DIC_WITHPARA = {"Y":"USE", "N":"NOT USE"}

DEF_SETTINGS_FILE = "utsusemi_installation_setting.json"

# keys of parameter dictionary
KEY_MLFSOFT_PATH = "mlfsoft_path"
KEY_DATA_DIR = "data_dir"
KEY_USER_DIR = "user_dir"
KEY_INSTTYPE = "manyo_insttype"
KEY_NUMTH = "numth"
KEY_REV = "rev"
KEY_WITHPARA = "with_para"
KEY_DISTRIBUTION = "distribution"
KEY_INSTALLATION_PATH = "installation_path"
KEY_ROOTPATH = "root_path"
KEY_SETTINGS_FILE = "settings_file"

# distribution definitions
KEY_DIST_RH8 = "RH8"
KEY_DIST_RH9 = "RH9"
KEY_DIST_UBU22 = "Ubuntu22"
KEY_DIST_UBU24 = "Ubuntu24"

def checkOS():
    """Check the distribution

    Args:
    Returns:
        str: name of distribution "RH8", "RH9", "Ubuntu"

    """
    if os.path.exists("/etc/redhat-release"):
        with open("/etc/redhat-release", "r") as f:
            r = f.readline()
            r_list = r.split(" ")
            for i, rel in enumerate(r_list):
                if rel == "release":
                    p = r_list[i + 1]
                    break
            if p.find("8") == 0:
                return KEY_DIST_RH8
            elif p.find("9") == 0:
                return KEY_DIST_RH9
    elif os.path.exists("/etc/issue"):
        with open("/etc/issue", "r") as f:
            t = f.readline()
            v = t.split(' ')
            if v[0] == "Ubuntu":
                if v[1].find("24.04") == 0:
                    return KEY_DIST_UBU24
                elif v[1].find("22.04") == 0:
                    return KEY_DIST_UBU22
            elif v[0] == "Linux" and v[1] == "Mint":
                ver_list = v[2].split(".")
                if int(ver_list[0]) == 22:
                    return KEY_DIST_UBU24
                elif int(ver_list[0]) == 21:
                    return KEY_DIST_UBU22

    print("ERROR OS is invalid. Avairable OS : RH(AlamaLinux, RockyLinux) 8/9 and UbuntuLinux 24.04/22.04 (Mint linux 22/21)")
    return ""

def parseArgs(param_dic):
    '''Parse argument of this script file

    Args:
        param_dic (dic): Dictionary to store parameters
    Returns:

    '''
    parser = argparse.ArgumentParser(description="The script to install Manyo/Utsusemi")
    parser.add_argument('--mlfsoft-path', help='set MLFSOFT_PATH')
    parser.add_argument('--manyo-insttype', help='set manyo-lib installation type "O"ld or "N"ew', default="O")
    parser.add_argument('--numth', type=int, help='set the number of multi-threading', default=2)
    parser.add_argument('--rev', help='revision number default:YYYYMMDD')
    parser.add_argument('--with-para', action='store_true', help='make with option -j')
    parser.add_argument('--without-para', action='store_true', help='make without option -j')
    args = parser.parse_args()

    if os.path.exists(DEF_SETTINGS_FILE):
        with open(DEF_SETTINGS_FILE, "r") as f:
            try:
                pre_params = json.load(f)
            except:
                pre_params = {}
            for akey in pre_params:
                param_dic[akey] = pre_params[akey]

    if len(param_dic) != 10:
        param_dic[KEY_MLFSOFT_PATH] = ""
        param_dic[KEY_INSTTYPE] = "O"
        param_dic[KEY_NUMTH] = 2
        param_dic[KEY_REV] = ""
        param_dic[KEY_WITHPARA] = "N"
        param_dic[KEY_DISTRIBUTION] = ""
        param_dic[KEY_ROOTPATH] = os.path.dirname(__file__)
        param_dic[KEY_USER_DIR] = os.path.join(os.environ["HOME"], "utsusemi")
        param_dic[KEY_DATA_DIR] = os.path.join(os.environ["HOME"], "mlfsoft", "data")
        param_dic[KEY_SETTINGS_FILE] = DEF_SETTINGS_FILE

    param_dic[KEY_MLFSOFT_PATH] = os.path.join(os.environ["HOME"], "mlfsoft")
    if args.mlfsoft_path is not None:
        param_dic[KEY_MLFSOFT_PATH] = args.mlfsoft_path
    param_dic[KEY_INSTTYPE] = "O"
    if args.manyo_insttype is not None:
        param_dic[KEY_INSTTYPE] = args.manyo_insttype
    param_dic[KEY_NUMTH] = 2
    if args.numth is not None:
        param_dic[KEY_NUMTH] = args.numth
    param_dic[KEY_REV] = ""
    if args.rev is not None:
        param_dic[KEY_REV] = args.rev
    param_dic[KEY_WITHPARA] = "N"
    if args.with_para and (not args.without_para):
        param_dic[KEY_WITHPARA] = "Y"
    param_dic[KEY_DISTRIBUTION] = checkOS()
    print("#[inamura 260717] CheckOS = {}".format(checkOS()))

def installCppPackages(params):
    """installation of packages for C++ libraries required by Manyo/Utsusemi

    Args:
        params (dictionary): Dictionary including some parameters produced by parseArgs()
    Returns:
        bool: True means succeeded.
    """
    com_list = []
    target_os = params[KEY_DISTRIBUTION]
    if target_os == KEY_DIST_RH8:
        #com_list.append('sudo yum groupinstall base "Development tools" --setopt=group_package_types=mandatory,default,optional')
        #com_list.append("sudo yum install epel-release")
        com_list.append("sudo dnf install -y swig cmake git")
        com_list.append("sudo dnf install -y mxml-devel hdf5-devel gsl-devel boost-devel openssl-devel")
        com_list.append("sudo dnf install -y blas-devel lapack-devel")
        com_list.append("sudo dnf install -y python39-devel")
        com_list.append("sudo dnf install -y python39-pip")
        com_list.append("sudo alternatives --set python3 /usr/bin/python3.9")
    elif target_os == KEY_DIST_RH9:
        com_list.append("sudo dnf install -y swig cmake git")
        com_list.append("sudo dnf install -y mxml-devel hdf5-devel gsl-devel boost-devel openssl-devel")
        com_list.append("sudo dnf install -y blas-devel lapack-devel")
        com_list.append("sudo dnf install -y python3.11-devel")
        com_list.append("sudo dnf install -y python3.11-pip")
        com_list.append("sudo alternatives --install /usr/bin/python3 python3 /usr/bin/python3.11 30")
        com_list.append("sudo alternatives --set python3 /usr/bin/python3.11")
        # com_list.append("sudo alternatives --set python3 /usr/bin/python3")
    elif target_os == KEY_DIST_UBU22 or target_os == KEY_DIST_UBU24:
        com_list.append("sudo apt-get install -y g++ swig make cmake")
        com_list.append("sudo apt-get install -y python3-dev")
        com_list.append("sudo apt-get install -y libhdf5-dev libmxml-dev libgsl-dev libssl-dev liblapack-dev")
        com_list.append('sudo apt-get install -y libnexus-dev')
        #com_list.append('sudo apt-get install -y libboost-dev')
        com_list.append('sudo apt-get install -y libboost-serialization-dev')
        com_list.append('sudo apt-get install -y libboost-filesystem-dev')
        com_list.append('sudo apt-get install -y libboost-system-dev')
        com_list.append('sudo apt-get install -y libboost-program-options-dev')
        com_list.append('sudo apt-get install -y zlib1g-dev')
        com_list.append('sudo apt-get install -y python3-pip')
        com_list.append('sudo apt-get install -y libxcb-cursor0')
    else:
        print("ERROR : Invalid os, which should be RH8/RH9/Ubuntu24/Ubuntu22/Mint22/Mint21")
        return False
    print("\n ==============================\n")
    for a_com in com_list:
        print("COMMAND : {}".format(a_com))
        if os.system(a_com) != 0:
            print("## ERROR : command = {}".format(a_com))
            return False
        else:
            pass

    if target_os == KEY_DIST_RH8 or target_os == KEY_DIST_RH9:
        if params[KEY_INSTTYPE] == "O":
            com_list = commandsBuildNexusformat(os.path.join(params[KEY_MLFSOFT_PATH], "manyo"))
        else:
            com_list = commandsBuildNexusformat(params[KEY_MLFSOFT_PATH])
        for a_com in com_list:
            print("COMMAND : {}".format(a_com))
            if os.system(a_com) != 0:
                print("## ERROR : command = {}".format(a_com))
                return False
            else:
                pass
    return True

def commandsBuildNexusformat(install_path):
    '''Make the installation proceedure of nexusformat from codes

    Args:
        install_path (str): The path to install nexusformat
    Returns:
        list: list of commands to install nexusformat

    '''
    com = []
    if not os.path.exists(os.path.join(os.getcwd(), "src", "code")):
        com.append("cd src;git clone https://github.com/nexusformat/code")
    else:
        com.append("rm -rf src/code/build")
    com.append("mkdir -p src/code/build")
    com.append("cd src/code/build;rm -rf *; cmake -DCMAKE_INSTALL_PREFIX={} ..".format(install_path))
    com.append("cd src/code/build; make; make install")
    com.append("mkdir -p {}".format(os.path.join(install_path, "lib")))
    com.append("cp {}/lib* {}".format(os.path.join(install_path, "lib64"), os.path.join(install_path, "lib")))
    return com

def commandsCMake(source_path, params):
    '''Make the build procedure of C++(Manyo) codes using CMake

    Args:
        source_path (str): Path to source codes folder
    Returns:
        list:  list of commands to build C++ codes using CMake
    '''
    install_path = params[KEY_MLFSOFT_PATH]
    numth = int(params[KEY_NUMTH])
    insttype = params[KEY_INSTTYPE]
    if insttype =="O":
        # install_path = os.path.join(install_path, "manyo")
        install_path = install_path
    elif insttype == "N":
        install_path = os.path.join(install_path, "utsusemi")

    utsusemi_rev = "0"
    print(" #################### Utsusemi LASTUPDATED = {}".format(os.path.join(os.path.join(params[KEY_ROOTPATH], "src", "LASTUPDATED"))))
    if os.path.exists(os.path.join(os.path.join(params[KEY_ROOTPATH], "src", "LASTUPDATED"))):
        with open(os.path.join(os.path.join(params[KEY_ROOTPATH], "src", "LASTUPDATED")), "r") as f0:
            utsusemi_rev = f0.readline().strip()
    print(" #################### Utsusemi revision = {}".format(utsusemi_rev))
    com = []
    build_path = os.path.join(source_path, "build")
    com.append("mkdir -p {}".format(build_path))
    com_cmake = "cd {};rm -rf *; cmake .. -DCMAKE_INSTALL_PREFIX={} -DMULTH={} -DMANYO_INSTTYPE={} -DUTSUSEMI_REV={}".format(build_path, install_path, numth, insttype, utsusemi_rev)
    if params[KEY_DISTRIBUTION] == KEY_DIST_RH8:
        com_cmake += ' -DPython_EXECUTABLE="/usr/bin/python3.9"'
    com.append(com_cmake)

    if params[KEY_WITHPARA] == "Y":
        com.append("cd {}; make -j install".format(build_path))
    else:
        com.append("cd {}; make install".format(build_path))

    params[KEY_INSTALLATION_PATH] = install_path
    return com

def execute(coms, outlist, errlist):
    '''Execute commands procedure given as str list.

    Args:
        coms (list): str list for commands
        outlist (list): list to store executed commands
        errlist (list): list to store error message by executing command
    returns:

    '''
    for acom in coms:
        print("EXECUTE : {}".format(acom))
        if os.system(acom) != 0:
            outlist.append(acom)
            errlist.append("ERROR on executing {}".format(acom))
            print("{} is failed.".format(acom))
            return False
        else:
            outlist.append(acom)
            errlist.append("")
    return True

def commandsPythonPackages():
    '''Make command procedure to install python packages using pip

    Args:
    Returns:
        list: list of commands
    '''
    coms = []
    target_os = params[KEY_DISTRIBUTION]
    if target_os == KEY_DIST_RH8:
        coms.append("python3 -m pip install numpy scipy matplotlib PySide2")
    elif target_os != KEY_DIST_UBU24:
        coms.append("python3 -m pip install numpy scipy matplotlib PySide6")
    return coms

def getLibraryInstallationPath(params):
    """Get the path of the folder of the installed libraries

    Args:
        params (dict) : parameters dictionary
    Returns:
        str: path to the installed librarirs folder
    """
    insttype = params[KEY_INSTTYPE]
    if insttype == "O":
        lib_path = os.path.join(params[KEY_MLFSOFT_PATH], "manyo", "lib")
    elif insttype == "N":
        py_ver = "python{}.{}".format(sys.version_info.major, sys.version_info.minor)
        if checkOS().find("Ubuntu") == 0:
            pack_path = "dist-packages"
        else:
            pack_path = "site-packages"
        lib_path = os.path.join(params[KEY_MLFSOFT_PATH], "lib", py_ver, pack_path)
    else:
        print("ERROR : insttype is invalid {}".format(insttype))
        return None
    return lib_path

def commandsPythonUtsusemi(source_path, params, target_bl=[]):
    '''Make command procedure to install python codes for Utsusemi

    Args:
        source_path (str): Path to source codes folder
        params (dic): Parameters dictionary
        target_bl (list): list of BL to install
    Returns:
        list: list of commands
    '''
    insttype = params[KEY_INSTTYPE]
    if insttype == "O":
        install_path = os.path.join(params[KEY_MLFSOFT_PATH], "python-utsusemi", "utsusemi")
    elif insttype == "N":
        py_ver = "python{}.{}".format(sys.version_info.major, sys.version_info.minor)
        if checkOS().find("Ubuntu") == 0:
            pack_path = "dist-packages"
        else:
            pack_path = "site-packages"
        install_path = os.path.join(params[KEY_MLFSOFT_PATH], "lib", py_ver, pack_path, "utsusemi")
    else:
        print("ERROR : insttype is invalid {}".format(insttype))
        return []

    # Utsusemi Base codes
    com = []
    com.append("mkdir -p {}".format(install_path))
    for sfolder in ["ana", "bin", "facade", "vis"]:
        com.append("cp -r {} {}".format(os.path.join(source_path, "Utsusemi", "python-utsusemi", sfolder), os.path.join(install_path)))
    com.append("cp {} {}".format(os.path.join(source_path, "Utsusemi", "python-utsusemi", "__init__.py"), os.path.join(install_path)))
    com.append("mkdir -p {}".format(os.path.join(params[KEY_USER_DIR], "ana", "tmp")))
    com.append("mkdir -p {}".format(os.path.join(params[KEY_USER_DIR], "ana", "xml")))

    # uGao
    com.append("cp -r {} {}".format(os.path.join(source_path, "uGao"), os.path.dirname(install_path)))

    # BL codes
    for a_bl in target_bl:
        print("commandsPythonUtsusemi target_bl = '{}'".format(a_bl))
        if a_bl in ["AMR", "HPN", "SIK", "VNR"]:
            print("Add {}".format(a_bl))
            com.append("cp -r {} {}".format(os.path.join(source_path, "Utsusemi", "python-utsusemi-{}".format(a_bl)), os.path.join(install_path, a_bl)))
        elif a_bl == "SAS":
            com.append("cp -r {} {}".format(os.path.join(source_path, "Utsusemi-SAS", "python-utsusemi-SAS"), os.path.join(install_path, "SAS")))
            com.append("cp {} {}".format(os.path.join(source_path, "Utsusemi-SAS", "python-utsusemi-SAS", "bin", "*"), os.path.join(install_path, "bin")))
        elif a_bl == "DNA":
            com.append("cp -r {} {}".format(os.path.join(source_path, "Utsusemi-DNA", "python-utsusemi-DNA"), os.path.join(install_path, "DNA")))
            com.append("cp -r {} {}".format(os.path.join(source_path, "Utsusemi-DNA", "environ-home"), os.path.join(params[KEY_USER_DIR], "ana", "DNA")))
        elif a_bl == "ENG":
            com.append("cp -r {} {}".format(os.path.join(source_path, "Utsusemi-ENG", "python-utsusemi-ENG"), os.path.join(install_path, "ENG")))
        else:
            print("ERROR : given BL {} is invalid.".format(a_bl))
            return None
    return com

def commandsMakeSetting(params):
    """Create commands to make setting to launch Utsusemi

    Args:
        params (dic): Parameters dictionary
    Returns:
        list: commands
    """
    com = []
    bashrc_path = os.path.join(params[KEY_MLFSOFT_PATH], "bashrc_utsusemi")
    title = "# Utsusemi environment setting\n"
    com.append("echo '{}' > {}".format(title, bashrc_path))
    setting1 = "export MLFSOFT_PATH={}\n".format(params[KEY_MLFSOFT_PATH])
    com.append("echo '{}' >> {}".format(setting1, bashrc_path))
    setting2 = "export PATH=${{MLFSOFT_PATH}}/{}:${{PATH}}\n".format(os.path.join("python-utsusemi", "utsusemi", "bin"))
    com.append("echo '{}' >> {}".format(setting2, bashrc_path))
    return com

def createUtsusemiEnvCfg(params, inst="SIK"):
    """
    """
    utsusemi_config = """
UTSUSEMI_INST_CODE={}
UTSUSEMI_LOG_QUIET=Y
UTSUSEMI_DEBUGMODE=N
UTSUSEMI_MULTH={}
UTSUSEMI_DATA_DIR="/home/yinamura/data"
UTSUSEMI_USR_DIR={}
UTSUSEMI_WORK_DIR={}
UTSUSEMI_DEFAULT_FONTSIZE=12
""".format(inst, params[KEY_NUMTH], params[KEY_USER_DIR], params[KEY_USER_DIR])
    # Utsusemi Environment
    # "utsusemienv.cfg"
    utsusemienv_cfg_dir = os.path.join(os.environ["HOME"], ".mlfsoft", "utsusemi")
    utsusemienv_cfg_path = os.path.join(utsusemienv_cfg_dir, "utsusemienv.cfg")
    com = "mkdir -p {}".format(utsusemienv_cfg_dir)
    if os.system(com) != 0:
        return False
    with open(utsusemienv_cfg_path, "w") as f:
        f.write(utsusemi_config)
    return True

def commandsDeleteAll(params):
    """Create commands to delete all

    Args:
        params (dic): Parameters dictionary
    Returns:
        list: commands
    """
    com = []
    if params[KEY_INSTTYPE] == "O":
        com.append("sudo rm -rf {}".format(os.path.join(params[KEY_MLFSOFT_PATH], "manyo")))
        com.append("sudo rm -rf {}".format(os.path.join(params[KEY_MLFSOFT_PATH], "python-utsusemi")))
    else:
        com.append("sudo rm -rf {}".format(os.path.join(params[KEY_MLFSOFT_PATH], "utsusemi")))
    return com


def MainMenu(params, outlist=[], errlist=[]):
    """Show the main menu

    Args:
        params (dic): Parameters dictionary
        outlist (list): executed commands
        errlist (list): occured errors
    Returns:
        bool: result for executing commands
    """
    menu_str = """
==================================================
     Welcome to Utsusemi software installer
==================================================
 Installation Directories
    MLF software root   : {}
    RAW data Directory  : {}
    User's directory    : {}
----------------------------------------------------------
Main menu :
1 .. Install ALL modules (all beamlines)
--------
2 .. Install Manyo Library and Utsusemi codes
3 .. Add Beamline environment

5 .. Change Installation Directories
7 .. Uninstall
0 or 9 .. Exit
----------------------------------------------------------
"""
    menu_str = """
==================================================
     Welcome to Utsusemi software installer
==================================================
 Installation Parameters
    MLF software root   : {}
    User's directory    : {}
    Number of threads   : {}
    Build with the parallel processing : {}

----------------------------------------------------------
Main menu :
1 .. Install ALL modules (all beamlines)
--------
2 .. Install Manyo Library and Utsusemi codes
3 .. Add Beamline Environment

5 .. Change Installation Parameters
7 .. Uninstall
0 or 9 .. Exit
----------------------------------------------------------
"""

    while(True):
        #print(menu_str.format(params[KEY_MLFSOFT_PATH], params[KEY_DATA_DIR], params[KEY_USER_DIR]))
        print(menu_str.format(params[KEY_MLFSOFT_PATH], params[KEY_USER_DIR], params[KEY_NUMTH], DIC_WITHPARA[params[KEY_WITHPARA]]))
        ret = input(">> Which ? ")
        if ret not in ["0", "1", "2", "3", "5", "7", "9"]:
            print("Exit without installation.")
            return False
        try:
            no = int(ret)
        except:
            print("Exit without installation.")
            return False
        result = False
        if no == 1:
            result = menu_1(params, outlist, errlist)
        elif no == 2:
            if inputMULTH(params):
                result = menu_2(params, ["manyo", "Utsusemi"], outlist, errlist)
        elif no == 3:
            result = menu_3(params, [], outlist, errlist)
        elif no == 5:
            result = menu_5(params)
        elif no == 7:
            ret = input(">>> All delete OK ? (N/yes): ")
            ret = ret.strip().upper()
            if ret == "YES":
                coms = commandsDeleteAll(params)
                if not execute(coms, outlist, errlist):
                    return False
        elif (not result) or no == 0 or no == 9:
            break
        with open(DEF_SETTINGS_FILE, "w") as f:
            json.dump(params, f)

        ret = input(">> Back to Main Menu? [Y/n]")
        if ret.strip().upper() == "N":
            break
    return True


def inputMULTH(params):
    """Input number of multi-threads

    Args:
        params (dic): Parameters dictionary
    Returns:
        bool: result for executing commands
    """
    ret = input(">>> How many threads for multi-treading data treatment (>=1, Default={}) :".format(params[KEY_NUMTH]))
    if ret == "":
        pass
    else:
        try:
            params[KEY_NUMTH] = int(ret)
        except:
            print("ERROR : input is invalid.")
            return False
    return True

def menu_1(params, outlist=[], errlist=[]):
    """Installation All

    Args:
        params (dic): Parameters dictionary
        outlist (list): executed commands
        errlist (list): occured errors
    Returns:
        bool: result for executing commands
    """
    if inputMULTH(params):
        if menu_2(params, SOURCE_ALL_LIST, outlist, errlist):
            if menu_3(params, ALL_BL_LIST, outlist, errlist):
                print("--------------------------------------------")
                print("Utsusemi Default Installations is completed.")
                print("--------------------------------------------")
                return True
    return False


def menu_2(params, target_list, outlist=[], errlist=[]):
    """Do Install Manyo Library and Utsusemi codes

    Args:
        params (dic): Parameters dictionary
        target_list (list): Build targets
        outlist (list): executed commands
        errlist (list): occured errors
    Returns:
        bool: result for executing commands
    """
    isCppPackInstalled = False
    if os.path.exists(os.path.join(getLibraryInstallationPath(params), "libUtsusemi.so")):
        isCppPackInstalled = True
    else:
        isCppPackInstalled = installCppPackages(params)

    if isCppPackInstalled:
        print("######### Cpp Packageses installation is succeeded.")
        for a_target in target_list:
            source_path = os.path.join(params[KEY_ROOTPATH], "src", a_target)
            if a_target in SOURCE_ALL_LIST:
                coms = commandsCMake(source_path, params)
            else:
                print("WARNING : Target dir = {} is not found.".format(a_target))
                print("          Available target Dir = ", SOURCE_ALL_LIST)
                return False
            for a_com in coms:
                print("com = {}".format(a_com))
            if not execute(coms, outlist, errlist):
                return False

        print("\nManyo Library and Utsusemi codes installation is completed.")
        print("-----------------------------------------------------------")
        return True
    else:
        return False

def menu_3(params, target_list=[], outlist=[], errlist=[]):
    """Add Beamline environment

    Args:
        params (dic): Parameters dictionary
        target_list (list): Targets to be installed
        outlist (list): executed commands
        errlist (list): occured errors
    Returns:
        bool: result for executing commands
    """
    # if never done menu #2, return False.
    if not os.path.exists(os.path.join(getLibraryInstallationPath(params), "libUtsusemi.so")):
        print("Menu #2 (Install Manyo Library and Utsusemi base codes) has never been executed.")
        print("You must do menu #2 befor doing menu #3.")
        return False

    menu2_str = """
---------------------------------
  Select Beam Lines you install
---------------------------------
 [ 1] BL01 4SEASONS (SIK)
 [ 2] BL02 DNA (DNA)
 [11] BL11 PLANET (HPN)
 [14] BL14 AMATERAS (AMR)
 [15] BL15 Taikan (SAS)
 [17] BL17 Sharaku (VNR)
 [19] BL19 Takumi (ENG)
----------------
 [ 0] Exit menu
---------------------------------
"""
    if len(target_list) == 0:
        print(menu2_str)
        ret = input(">>> Which Packages (N[, M, L]): ")
        for a_bl in ret.split(","):
            if a_bl.strip() == "0":
                target_list = []
                break
            if a_bl in DICT_BLCODE:
                target_list.append(DICT_BLCODE[a_bl.strip()])
    if len(target_list) == 0:
        return False

    result = True
    installed_list = ""
    for a_bl in target_list:
        if a_bl == "SAS":
            if not os.path.exists(os.path.join(getLibraryInstallationPath(params), "libSAS.so")):
                if inputMULTH(params):
                    result = menu_2(params, ["Utsusemi-SAS"], outlist, errlist)
        elif a_bl == "DNA":
            if not os.path.exists(os.path.join(getLibraryInstallationPath(params), "libDNA.so")):
                if inputMULTH(params):
                    result = menu_2(params, ["Utsusemi-DNA"], outlist, errlist)
        elif a_bl == "ENG":
            if not os.path.exists(os.path.join(getLibraryInstallationPath(params), "libEmaki.so")):
                if inputMULTH(params):
                    result = menu_2(params, ["Utsusemi-ENG"], outlist, errlist)
        installed_list += (a_bl + ",")

    if not result:
        return False

    coms = commandsPythonPackages()
    if not execute(coms, outlist, errlist):
        return False

    source_path = os.path.join(params[KEY_ROOTPATH], "src")
    coms = commandsPythonUtsusemi(source_path, params, target_list)
    if len(coms) != 0:
        if not execute(coms, outlist, errlist):
            return False

    coms = commandsMakeSetting(params)
    if len(coms) != 0:
        if not execute(coms, outlist, errlist):
            return False

    if createUtsusemiEnvCfg(params, target_list[0]):
        pass
    else:
        print("# ERROR : Failed to write utsusemienv.cnf")
        return False

    print("\nAddition of {} environments is completed.".format(installed_list[:-1]))
    print("-----------------------------------------------------------")
    return True


def menu_5(params):
    """Change Installation parameters

    Args:
        params (dic): Parameters dictionary
    Returns:
        bool: result for executing commands
    """
    menu5_str = """
Change Installation Directories
--------------------------------------
1 .. MLF software root   : {}
2 .. RAW data Directory  : {}
3 .. User's directory    : {}
0 .. Return to main menu
"""
    menu5_str = """
Change Installation Parameters
--------------------------------------
1 .. MLF software root   : {}
2 .. User's directory    : {}
3 .. Number of threads   : {}
4 .. Build with the parallel processing : {}
0 .. Return to main menu
"""
    ans = -1
    while(ans != 0):
        # print(menu5_str.format(params[KEY_MLFSOFT_PATH], params[KEY_DATA_DIR], params[KEY_USER_DIR]))
        print(menu5_str.format(params[KEY_MLFSOFT_PATH], params[KEY_USER_DIR], params[KEY_NUMTH], DIC_WITHPARA[params[KEY_WITHPARA]]))
        ret = input(">>> Which change: ")
        try:
            ans = int(ret)
        except:
            ans = 0
        if ans == 1:
            ret = input(">>> new MLF software root: ")
            if ret != "":
                params[KEY_MLFSOFT_PATH] = ret.strip()
        elif ans == 2:
            ret = input(">>> new users dir: ")
            if ret != "":
                params[KEY_USER_DIR] = ret.strip()
        elif ans == 3:
            ret = input(">>> new number of threads: ")
            if ret != "":
                try:
                    params[KEY_NUMTH] = int(ret.strip())
                except:
                    print("Invalid Number.")
        elif ans == 4:
            ret = input(">>> Parallel processing [y/n]: ")
            ret = ret.strip().upper()
            if ret == "Y":
                params[KEY_WITHPARA] = "Y"
            else:
                params[KEY_WITHPARA] = "N"

        """
        elif ans == 2:
            ret = input(">>> new RAW data dir: ")
            if ret != "":
                params[KEY_DATA_DIR] = ret.strip()
        else:
            pass
        """
    return True

if __name__ == '__main__':
    params = {}
    parseArgs(params)
    outlist = []
    errlist = []

    if MainMenu(params, outlist, errlist):
        print("Finished.")
        # print(outlist)
    else:
        print("ERROR occurs.")
        # print(errlist)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function

import utsusemi.SAS.ana.Reduction.CorrBaseCommands as CBC

# special reserved word for commands history
HISTORY = None

# reserved words for return value in this.
DAT = None
ecs = None
EC = None
ret = None
PyObj = None
Iq = None
Tr = None


########################################
def NormByMonCounts( dat=DAT, monitor=EC, key="Lamb", XRange="0.6 7.6", factor=1.0 ):
    """
    Normalize data by monitor counts
    @param  dat      (ElementContainerMatrix, ElementContainer) target data
    @param  monitor  (ElementContainer) Monitor data from GetMonHistSAS
    @param  Xrange   (string)  "min-value max-value" for the range to sumalize intensities.
    @param  factor   (float)   factor for normalization I=(Iorg/mon)*factor
    @return None
    """
    CBC.NormByMonCounts( dat, monitor, key, XRange, factor)

########################################
def CorrectDetEffi( dat=DAT, filename="detectorEffiTBL.dat" ):
    """
    Does the correction of detector efficiency for lambda
    CorrectDetEffi( dat, filename )
    
    @param  dat       (ElementContainerMatrix)
    @param  filename  (string)
    @retval None
    """
    print("#######")
    CBC.CorrectDetEffi( dat, filename)

#########################################
def CorrectDetEffiRPMT( dat=DAT, alpha=0.10516, beta=0.04838 ):
    """
    Does the correction of detector efficiency for lambda
    CorrectDetEffi( dat, filename )
    
    @param  dat   (ElementContainerMatrix)
    @param  alpha (float) parameter of lambda dependency for detect efficiency of RPMT
    @param  beta  (float) parameter of lambda dependency for detect efficiency of RPMT
    @retval None
    """
    CBC.CorrectDetEffiRPMT( dat, alpha, beta )

#########################################
def I0LambdaCorrection(dat=DAT,I0Lamb=EC):
    """
    I0(lambda) correction
    @param dat      (ElementContainerMatrix)
    @param I0Lamb   (ElementContainer)
    """
    CBC.I0LambdaCorrection(dat, I0Lamb)

#########################################
def NormByKickers(dat=DAT):
    """
    Normalize Intensity by the number of kickers
    @param dat   (ElementContainerMatrix)
    @retval None
    """
    CBC.NormByKickers(dat)

# Dictionary for entry the name of functions
_functions = {
              "NormByMonCounts":NormByMonCounts,
              "CorrectDetEffiRPMT":CorrectDetEffiRPMT,
              "CorrectDetEffi":CorrectDetEffi,
              "I0LambdaCorrection":I0LambdaCorrection,
              "NormByKickers":NormByKickers
}




#! /usr/bin/env python
# -*- coding:utf-8 -*-
import datetime
import time
import os
import sys
import shutil
import Manyo as mm
import Manyo.Utsusemi as mu
import Cmm
import uGao.MPlot as MP
import uGao.M2PlotPlusCui as MP2
import ElasticScanGUIBase as ESB
import T0fileMake as T0M
import subprocess

import ElasticScanPy3 as ESP3

PYSIDEVER = 6
try:
    from PySide6 import QtCore, QtGui, QtWidgets
    from PySide6.QtWidgets import QApplication, QFileDialog, QListWidgetItem, QMessageBox
    from PySide6.QtCore import QFile
    from ui6_ElasticScanGUIQt import Ui_MainWindow
except:
    from PySide2 import QtCore, QtGui, QtWidgets
    # from PySide2.QtUiTools import QUiLoader
    from PySide2.QtWidgets import QApplication, QFileDialog, QListWidgetItem, QMessageBox
    from PySide2.QtCore import QFile
    from ui2_ElasticScanGUIQt import Ui_MainWindow
    PYSIDEVER = 2

class App(QtWidgets.QMainWindow):
    def __init__(self, master = None, ):
        super(App, self).__init__(master)
        self.frame = Ui_MainWindow()
        self.frame.setupUi(self)

        bkg = self.findChild(QtWidgets.QWidget, 'background')
        bkg.setStyleSheet("background-color: cyan")

        self.run = self.findChild(QtWidgets.QSpinBox, 'run')
        self.readtemp_bt = self.findChild(QtWidgets.QPushButton, 'readtemp_bt')
        self.Sta = self.findChild(QtWidgets.QDoubleSpinBox, 'Sta')
        self.End = self.findChild(QtWidgets.QDoubleSpinBox, 'End')
        self.End_2 = self.findChild(QtWidgets.QDoubleSpinBox, 'End_2')
        self.Bin = self.findChild(QtWidgets.QDoubleSpinBox, 'Bin')
        self.Ti = self.findChild(QtWidgets.QDoubleSpinBox, 'Ti')
        self.Tf = self.findChild(QtWidgets.QDoubleSpinBox, 'Tf')
        self.TBin = self.findChild(QtWidgets.QDoubleSpinBox, 'TBin')
        self.rb1_NFinished = self.findChild(QtWidgets.QRadioButton, 'rb1_NFinished')
        self.rb1_Finished = self.findChild(QtWidgets.QRadioButton, 'rb1_Finished')
        self.rb2_FCOFF = self.findChild(QtWidgets.QRadioButton, 'rb2_FCOFF')
        self.rb2_FCON_11_225 = self.findChild(QtWidgets.QRadioButton, 'rb2_FCON_11_225')
        self.rb2_FCON_33_225 = self.findChild(QtWidgets.QRadioButton, 'rb2_FCON_33_225')
        self.rb2_FCON_1S_225 = self.findChild(QtWidgets.QRadioButton, 'rb2_FCON_1S_225')
        self.rb2_FCON_33_300 = self.findChild(QtWidgets.QRadioButton, 'rb2_FCON_33_300')
        self.rb2_FCON_11_300 = self.findChild(QtWidgets.QRadioButton, 'rb2_FCON_11_300')
        self.T0shift = self.findChild(QtWidgets.QSpinBox, 'T0shift')
        self.dqs1 = self.findChild(QtWidgets.QDoubleSpinBox, 'dqs1')
        self.emin = self.findChild(QtWidgets.QDoubleSpinBox, 'emin')
        self.emax = self.findChild(QtWidgets.QDoubleSpinBox, 'emax')
        self.de = self.findChild(QtWidgets.QDoubleSpinBox, 'de')
        self.rb3_CHOFF = self.findChild(QtWidgets.QRadioButton, 'rb3_CHOFF')
        self.rb3_CHON_11_225 = self.findChild(QtWidgets.QRadioButton, 'rb3_CHON_11_225')
        self.dqs3 = self.findChild(QtWidgets.QDoubleSpinBox, 'dqs3')
        self.emin3 = self.findChild(QtWidgets.QDoubleSpinBox, 'emin3')
        self.emax3 = self.findChild(QtWidgets.QDoubleSpinBox, 'emax3')
        self.de3 = self.findChild(QtWidgets.QDoubleSpinBox, 'de3')
        self.button1 = self.findChild(QtWidgets.QPushButton, 'button1')
        self.button3 = self.findChild(QtWidgets.QPushButton, 'button3')
        
        #Tkinter.Frame.__init__(self, master,bg="#00FFFF")
        #self.master.title("DNA Elastic Scan Reduction Check")

        #self.errorstr
# Set Run No.
        #self.run.insert( 0, '10371')
        self.run.setValue(10371)
        
        # Read Temperature
        ChList = ("A","B","C","C2","D","D2","D3","D4","D5","PEACE_U","PEACE_M","PEACE_L","PEACE_UB","PEACE_MB","PEACE_LB")
        #self.ChMenu
        self.ChMenu = self.findChild(QtWidgets.QComboBox, 'ChMenu')
        self.readtemp_bt.clicked.connect(self.readtemp)

        #####################
        #Set Time Parameter
        #self.Sta
        #self.End
        #self.Bin
        ###########
        #Set Tempaerature Parameter
        #self.Ti
        #self.Tf.insert( 0, '300')
        self.Tf.setValue(300)
        #self.TBin.insert( 0, '1')
        self.TBin.setValue(1)

        self.rb1_grp = QtWidgets.QButtonGroup()
        self.rb1_grp.addButton(self.rb1_NFinished, 1)
        self.rb1_grp.addButton(self.rb1_Finished, 2)
        self.rb1_grp.button(1).setChecked(True)
        

# Si(111) self.si < 4, Si(333) self.si > 3
# Si111 Select
        self.si = 0

        self.rb2_FCOFF.clicked.connect(self.sel_si111)
        self.rb2_FCON_11_225.clicked.connect(self.sel_si111)
        self.rb2_FCON_33_225.clicked.connect(self.sel_si111)
        self.rb2_FCON_1S_225.clicked.connect(self.sel_si111)
        self.rb2_FCON_33_300.clicked.connect(self.sel_si111)
        self.rb2_FCON_11_300.clicked.connect(self.sel_si111)
        self.v111 = QtWidgets.QButtonGroup()
        self.v111.addButton(self.rb2_FCOFF, 1)
        self.v111.addButton(self.rb2_FCON_11_225, 2)
        self.v111.addButton(self.rb2_FCON_33_225, 3)
        self.v111.addButton(self.rb2_FCON_1S_225, 4)
        self.v111.addButton(self.rb2_FCON_33_300, 5)
        self.v111.addButton(self.rb2_FCON_11_300, 6)

        self.rb3_CHOFF.clicked.connect(self.sel_si333)
        self.rb3_CHON_11_225.clicked.connect(self.sel_si333)
        #self.v333 = QtWidgets.QButtonGroup()
        #self.v333.addButton(self.rb3_CHOFF, 5)
        #self.v333.addButton(self.rb3_CHON_11_225, 6)
        self.v111.addButton(self.rb3_CHOFF, 7)
        self.v111.addButton(self.rb3_CHON_11_225, 8)
        
        self.dqs1.setValue(0.05)

        #self.v111.set(2)
        self.v111.button(3).setChecked(True)

        self.emin.setValue(-4)
        self.emax.setValue(4)
        self.de.setValue(2)
	
        #[20220403] T0shift Add. by TY 
        self.T0shift.setValue(12325)
        #self.label_22.setText()

#Si 333 GUI
        self.dqs3.setValue(0.15)
        #self.v333.set(0)
        #if self.v333.checkedButton() is not None:
        #    self.v333.checkedButton().setChecked(False)
        ##self.v333.button(1).setChecked(True)
        
        self.emin3.setValue(-500)
        self.emax3.setValue(100)
        self.de3.setValue(50)
        #rd = Tkinter.Label( f33, text="μeV")

        self.show()

# Button
        #button1 = Tkinter.Button(self, text = "Reduction Start", command = self.launch)
        #button3 = Tkinter.Button(self, text = "Quit", command = self.master.destroy)
        self.button1.clicked.connect(self.launch)
        self.button3.clicked.connect(self.closeThis)

# self.step == 10: launch stand-by
        self.step = 10

        self.FastChopperUse = True
        self.CommentHeader = "ElasticGUI ->>"

    def sel_si111(self):
        if self.step == 10:
            #i = self.v111.get()
            i = self.v111.checkedId() - 1
        if ( self.si != i):
            #self.v333.set(0)
            #if self.v333.checkedButton() is not None:
            #    self.v333.checkedButton().setChecked(False)
            self.si = i
            self.emin.setValue(0.0)
            self.emax.setValue(0.0)
            self.de.setValue(0.0)
        if ( i == 0):
            self.emin.setValue(-200)
            self.emax.setValue(50)
            self.de.setValue(50)
            self.FastChopperUse = False
            self.T0shift.setValue(120)
        elif ( i == 1):
            self.emin.setValue(-7)
            self.emax.setValue(5)
            self.de.setValue(3)
            self.FastChopperUse = True
            self.T0shift.setValue(12325)
        elif ( i == 2):
            self.emin.setValue(-4)
            self.emax.setValue(4)
            self.de.setValue(2)
            self.FastChopperUse = True
            self.T0shift.setValue(12325)
        elif ( i == 4):
            self.emin.setValue(-2)
            self.emax.setValue(2)
            self.de.setValue(2)
            self.FastChopperUse = True
            self.T0shift.setValue(12325)
        elif ( i == 5):
            self.emin.setValue(-2)
            self.emax.setValue(2)
            self.de.setValue(2)
            self.FastChopperUse = True
            self.T0shift.setValue(12275)
        else:
            self.emin.setValue(-10)
            self.emax.setValue(10)
            self.de.setValue(5)
            self.FastChopperUse = True
            self.T0shift.setValue(12325)

    def sel_si333(self):
        if self.step == 10:
            #i = self.v333.get()
            #i = self.v333.checkedId() - 1
            i = self.v111.checkedId() - 1
            if ( self.si != i):
                #self.v111.set(-1)
                #if self.v111.checkedButton() is not None:
                #    print("#[inamura 230301] setChecked(False) checked ID={}".format(self.v111.checkedId()))
                #    self.v111.button(2).setChecked(True)
                #    #self.v111.checkedButton().setChecked(False)
                self.si = i
                self.emin3.setValue(0)
                self.emax3.setValue(100)
                self.de3.setValue(50)
            if ( i == 6):
                self.emin3.setValue(-500)
                self.FastChopperUse = False
            else:
                self.emin3.setValue(-100)
                self.FastChopperUse = True

    def readtemp(self):
        if self.step == 10:
            #runNo = int(self.run.get())
            #TCh = self.Ch.get()
            runNo = int(self.run.value())
            TCh = self.ChMenu.currentText()
            #TCh = self.ChMenu.get()
            print (self.CommentHeader, "RunNo: DNA%6d" %(runNo))
            print (self.CommentHeader, "Temperature Sensor: %s" %(TCh))
            print (self.CommentHeader, "Reading Temperature Start....")
            EC = Cmm.CheckSampleTempByRunNumber( runNo,TCh)
            print (self.CommentHeader, "Reading Temperature end")
            MP.MPlot(EC)
            self.frame.errorstr.setText("Temperature run# %d Ch %s" % (runNo, TCh))

    def closeThis(self):
        self.close()

    def launch(self):
        self.SetParams()
        self.StartElasticScan() #temporal comment out

    def SetParams(self):
        #self.RunNo = int(self.run.get())
        #self.Tch = self.Ch.get()
        self.RunNo = self.run.value()
        self.Tch = self.ChMenu.currentText()
        print (self.CommentHeader, "Run No.:", self.RunNo )
        print (self.CommentHeader, "Temperature Sensor: %s" %(self.Tch))
        #self.StartTime = self.Sta.get()
        #self.EndTime = self.End.get()
        #self.TimeBin = self.Bin.get()
        self.StartTime = str(self.Sta.value())
        self.EndTime = str(self.End.value())
        self.TimeBin = str(self.Bin.value())
        print (self.CommentHeader, "StartTime: %s sec" %(self.StartTime))
        print (self.CommentHeader, "EndTime: %s sec" %(self.EndTime))
        print (self.CommentHeader, "TimeBin: %s sec" %(self.TimeBin))
        print (self.CommentHeader, "FastChopperUse:", self.FastChopperUse)
        #self.StartTemp = self.Ti.get()
        #self.EndTemp = self.Tf.get()
        #self.TempBin = self.TBin.get()
        self.StartTemp = str(self.Ti.value())
        self.EndTemp = str(self.Tf.value())
        self.TempBin = str(self.TBin.value())
        print (self.CommentHeader, "Start Temperature: %s K" %(self.StartTemp))
        print (self.CommentHeader, "End Temperature: %s K" %(self.EndTemp))
        print (self.CommentHeader, "Temperature BIn: %s K" %(self.TempBin))


        self.Analyzer = 111
        self.Emin = 0
        self.Emax = 0
        self.Ebin = 0
        self.Qmin = 0
        self.Qmax = 0
        self.Qbin = 0
        
        #Mode = self.v111.get()
        #Mode3 = self.v333.get()
        Mode = self.v111.checkedId() - 1
        Mode3 = self.v111.checkedId() - 3
        if Mode3 ==4 or Mode3 ==5:
            self.Analyzer = 333
        print (self.CommentHeader, "Analyzer:%d" %(self.Analyzer))
        
        if Mode <=5 and Mode>=0:
            self.Emin = self.emin.value()
            self.Emax = self.emax.value()
            self.Ebin = self.de.value()
            self.Qmax = 2.0
            self.Qbin = str(self.dqs1.value())
        elif Mode3 >=4:
            self.Emin = self.emin3.value()
            self.Emax = self.emax3.value() 
            self.Ebin = self.de3.value()
            self.Qmax = 6.0
            self.Qbin = str(self.dqs3.value())
        else:
            print (self.CommentHeader, "EBin Set Error")
            return -1
        
        self.Emin = float(self.Emin)/1000
        self.Emax = float(self.Emax)/1000
        self.Ebin = float(self.Ebin)/1000
        print (self.CommentHeader, "Emin:%.4f" %(self.Emin))
        print (self.CommentHeader, "Emax:%.4f" %(self.Emax))
        print (self.CommentHeader, "Ebin:%.6f" %(self.Ebin))
        print (self.CommentHeader, "Qmin:%2.1f" %(self.Qmin))
        print (self.CommentHeader, "Qmax:%2.1f" %(self.Qmax))
        print (self.CommentHeader, "Qbin:%s" %(self.Qbin))

    def showplot(self,dat):
        if type(dat) == type(mm.ElementContainer()):
            MP.MPlot(dat)
        elif type(dat) == type(mm.ElementCotainerArray()):
            MP2.M2Plot(dat)
        else:
            print (self.CommentHeader, "dat is not ElementContainer or ElementContainerArray")
            print (self.CommentHeader, "dat type is ", type(dat))

    def StartElasticScan(self):
        print (self.CommentHeader)
        print (self.CommentHeader, "ElasticScan Reduction Start")
        
        inifile = "ESGUIParams.ini"
        iniparams = ESB.SetESiniFile(inifile)
        
        iniparams.setparams("laticetype",self.Analyzer)
        if self.FastChopperUse:
            iniparams.setparams("UseFastChopper","True")
            if self.Analyzer == 111:
#                Mode = self.v111.get()
                #iniparams.setparams("t0_offset",self.T0shift.get())
                iniparams.setparams("t0_offset",str(self.T0shift.value()))
#                print (Mode)
#                if(Mode == 4):
#                    iniparams.setparams("t0_offset","12375")
#                elif(Mode == 5):
#                    iniparams.setparams("t0_offset","12275")
#                    #iniparams.setparams("t0_offset","12365")
#                else:
#                    iniparams.setparams("t0_offset","12325")
            elif self.Analyzer == 333:
                iniparams.setparams("t0_offset","4091")
        else:
            iniparams.setparams("UseFastChopper","False")
            #iniparams.setparams("t0_offset",self.T0shift.get())
            iniparams.setparams("t0_offset",str(self.T0shift.value()))
            #iniparams.setparams("t0_offset","120")
        
        iniparams.writeESinifiles()
        #---LambdaParam
        LambdaCenter = 6.321
        dLambda = 4.15 
        if self.Analyzer == 333:
            LambdaCenter = 3.4 
            dLambda = 4.16 
        
        LambdaParam = "%.4f,%.4f" %(LambdaCenter,dLambda)
        HWParam ="%.4f,%4f,%.6f" %(self.Ebin,self.Emin,self.Emax)
        HWParam2 ="%.4f,%4f" %(self.Emin,self.Emax)
        QParam = "%.2f,%.2f,%s" %(self.Qmin,self.Qmax,self.Qbin)
        print (self.CommentHeader, "HWParam: %s" %( HWParam))
        print (self.CommentHeader, "HWParam2: %s" %(HWParam2))
        print (self.CommentHeader, "LambdaParam: %s"  %(LambdaParam))
        print (self.CommentHeader, "Qparam: %s" %(QParam))
       
        TimeParam = "%s,%s,%s" %(self.StartTime,self.EndTime,self.TimeBin)
        TempParam = "%s,%s,%s" %(self.StartTemp,self.EndTemp,self.TempBin)
        print (self.CommentHeader, "TimeParam: %s" %(TimeParam))
        print (self.CommentHeader, "TempCh: %s" %(self.Tch))
        print (self.CommentHeader, "TempParam: %s" %(TempParam))

        dataPath = os.path.join(mu.UtsusemiEnvGetUserDir(), "DNA", "tmp")  #[inamura 230301]
        #dataPath = "/home/dnaadmin/DNA/tmp/" #T.Yamada[130218]
        ECMbstName = "ECM_%d_ES.bst" %(self.RunNo)
        
        FileHeader = "ESGUIRun"
        ESName = "%s%dES.txt" % (FileHeader, self.RunNo)
        ESTimeName = "%s%dU2time.txt" % (FileHeader, self.RunNo)
        ESTempName = "%s%dU2vsT.txt" % (FileHeader, self.RunNo)
        TIName = "%s%dTI.txt" % (FileHeader, self.RunNo)
        TITimeName = "%s%dTItime.txt" % (FileHeader, self.RunNo)
        TITempName = "%s%dTIvsT.txt" % (FileHeader, self.RunNo)
        TempLogName = "%s%dTempLog.txt" %(FileHeader, self.RunNo)
    
        #copyresult, messg =CopyEventData(int(self.RunNo))
        #print (messg) 

        #print (type(self.RunNo), self.RunNo)
        T0M.MakeT0file(str("%06d" %int(self.RunNo)))

        import ElasticScan 
        #Make Case Info xml
        casefname = ESP3.MakeCaseInfoXML(float(self.StartTime), float(self.EndTime), float(self.TimeBin))
        print (self.CommentHeader, "ECM make start")
        ECM = ESP3.ESTimeSlice(RunNo=int(self.RunNo), dataFolder="/data", HWParam=HWParam, LambdaParam=LambdaParam, QParam=QParam, inifile=inifile, cfile=casefname)
        #ECM = ElasticScan.GetQETimeECMMulth(self.RunNo, HWParam, LambdaParam, QParam, TimeParam, inifile)
        print (self.CommentHeader, "ECM make end")
        
        #ECM = ElasticScan.ReadECM(ECMbstName)
        EC = ElasticScan.ElasticScanTotalIntensity(ECM, HWParam2, QParam, inifile)
        print (self.CommentHeader, "Total Intensity vs Time")
        ElasticScan.OutputECText(EC, TITimeName)
        ElasticScan.OutPutElasticScanECM(ECM)
        #del ECM
        #del ElasticScan
        self.showplot(EC)
        print ("Finished")
        EC2 = Cmm.CheckSampleTempByRunNumber(self.RunNo,self.Tch)
        EC4 = ESP3.ConvertXaxisFromTimeToTemp(EC,EC2)
        ElasticScan.OutputECText(EC4, TITempName) #TotalIntensity vs T
        self.showplot(EC4)
        #u2 vs time is calculated
        ECA = ElasticScan.ElasticScanQ2LnI(ECM, HWParam2, QParam, inifile)
        EC1 = ElasticScan.ElasticScanFit(ECA, "p1")
        ElasticScan.OutputECText(EC1, ESTimeName) # U2 vs Time
        self.showplot(EC1)
        EC3 = ESP3.ConvertXaxisFromTimeToTemp(EC1,EC2)
        ElasticScan.OutputECText(EC3, ESTempName) #U2 vs T
        self.showplot(EC3)
        del ECM
        del ElasticScan
        del ECA
        print (self.CommentHeader, "ElasticScan Reduction End")
        print (self.CommentHeader, "I(Q) vs time is saved as \n%sRun%sIQ.txt\n" %(dataPath,self.RunNo) )
        print (self.CommentHeader, "u2 is saved as \n%s%s\n%s%s\n" %(dataPath, ESTempName, dataPath, ESTimeName))
        print (self.CommentHeader, "Total intensity is saved as \n%s%s\n%s%s\n" %(dataPath, TITempName, dataPath, TITimeName))
        #print (self.CommentHeader, "Temperature Log is saved as \n%s%s\n" %(dataPath, TempLogName))
        return -1
        

def CopyEventData(RunNo):
    command = "ssh dnaadmin@192.168.1.10 /home/dnaadmin/Yamada/01_CopyEventData/CopyEventEdata.sh %d" %(RunNo)
    args = command.split(" ")
    proc = subprocess.Popen(args, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
    stdout_data, stderr_data = proc.communicate()
    print ("stdout_data >>", stdout_data)
    print ("stderr_data >>", stderr_data.replace("\n",""))
    result = proc.returncode
    #proc.close()
    if result == 1:
        messg =  "DNA%06d_* is not found on /data and /ifs_data/DNA/DNA" %RunNo
    else:
        messg =  "DNA%06d_* is ready for the reduction." %RunNo
    return result, messg


if __name__ == "__main__":
    FastChopperUse = False
    if PYSIDEVER == 2:
        if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
            QtWidgets.QApplication.setAttribute(
                QtCore.Qt.AA_EnableHighDpiScaling, True)

    app = QApplication(sys.argv)
    win = App(None)
    if PYSIDEVER == 6:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



#! /usr/bin/env python3
import sys
import os
import argparse
import subprocess
################################################################################
# Compile script for Manyo-lib release version
#    ver 20240212
#    written by Y. Inamura
#########################################################################
"""
Usage :     python3 do_make_Manyo.py <command> [options]
       command: [depend/man/make/install/clean]
       options: --debug --help --prefix --manyo-path
                --other-modules --numth --pyver
                --rev --without-depend --with-para --without-para
                --oldinstalltype
  -------------------------------------------------------
  depend  : depend script
  man     : script to make manual
  make    : make
  install : make install
  clean   : make clean

  --debug          : debug mode
  --help           : show this help
  --prefix=/hoge/hogege/hoge
                   : set directory to install
  --manyo-path=/hoge/hogege/hoge
                   : set directory to install with old structures
  --dist-name=Ubuntu
                   : Linux distribution name [RH6,RH7,Ubuntu]
  --other-modules=
                   : required other modules if need

  --numth=n        : set the number of multi-threading
  --pyver=n.m      : set python version
                     If not set pyver, the version of python executing this is used
  --rev=nnnn       : set revision number if need
  --without-depend : does not run depend script
  --with-para      : use compile option of -j
  --without-para   : not use compile option of -j (default)
  --oldinstalltype : install to old path (/opt/mlfsfot/)

"""

#########################################################################
#   CXX            : compiler
#   PYVER          : version of python
#   NUM_OF_THREADS : number of threading
#   PREFIX         : path to install
#   OTHER_MANYO_PAC: Other Manyo Package names required by compile
#   ENV_FILE_NAME  : file name of shell script to set Manyo environment
#   EXT_LIB_ROOT   : root path to external library required by Manyo-lib
#   DIST_NAME      : name of linux distribution
#########################################################################
CXX = "g++"
# PYVER          = "%s.%s"%(sys.version_info[0],sys.version_info[1])
PYVER = ""
try:
    PYVER = "{:d}.{:d}".format(sys.version_info.major, sys.version_info.minor)
except:
    pass

NUM_OF_THREADS = 8
PREFIX = "/opt/mlfsoft/utsusemi"
OTHER_MANYO_PAC = ["Utsusemi"]
ENV_FILE_NAME = "bashrc.manyo.release"
EXT_LIB_ROOT = "/usr/local"
DIST_NAME = ""
FOUND_DIST_NAME = ""
if os.path.exists("/etc/redhat-release"):
    f = open("/etc/redhat-release", "r")
    r=f.readline()
    f.close()
    p = r.split(" ")[-2]
    if p > "7":
        FOUND_DIST_NAME = "RH7"
    else:
        FOUND_DIST_NAME = "RH6"
elif os.path.exists("/etc/os-release"):
    fo = open("/etc/issue", "r")
    t = fo.readline()
    fo.close()
    v = t.split(' ')
    if v[1].find("24.04") == 0 or v[1].find("22.04") == 0 or v[1].find("20.04") == 0 or v[1].find("19.") == 0:
        FOUND_DIST_NAME = "Ubuntu20"
    elif v[1].find("18.04") == 0 or v[1].find("17.") == 0 or v[1].find("16.04") == 0 or v[1].find("15.") == 0:
        FOUND_DIST_NAME = "Ubuntu16"
    print("## Ubuntu version = {}".format(DIST_NAME))
else:
    pass

isDebug = False
isWithDepend = True
isCompPara = False
isNotManyoCore = True
isOldInstallType = False
if sys.version_info.major==2:
    isOldInstallType = True

manyo_rev = 0
if "MANYO_REV" in os.environ:
    manyo_rev = int(os.environ["MANYO_REV"])
else:
    try:
        cp = subprocess.run("git log -1 --date=short|grep 'Date:'", shell=True, capture_output=True, encoding="utf-8")
        if cp.stderr == '':
            latest_date_git = cp.stdout.split()[1]
            latest_date_v = latest_date_git.split("-")
            manyo_rev = int(latest_date_v[0][-2:] + latest_date_v[1] + latest_date_v[2])
    except:
        pass

#############################################
# Check arguments
#############################################

do_command = ""

parser = argparse.ArgumentParser(description="The script to compile manyo library source codes")

parser.add_argument('command', choices=['depend', 'man', 'make', 'install', 'clean'],
                    help='command in [depend/man/make/install/clean]')
parser.add_argument('--prefix', help='set MLFSOFT_PATH')
parser.add_argument('--manyo-path', help='set MANYO_PATH')
parser.add_argument('--dist-name',
                    choices=["RH6", "RH7", "RH", "Ubuntu", "Ubuntu16", "Ubuntu18", "Ubuntu20", "Ubuntu22", "Ubuntu24"],
                    help="Disribution name")
parser.add_argument('--other-modules', help="Other modules required if need.")
parser.add_argument('--numth', type=int, help='set the number of multi-threading', default=NUM_OF_THREADS)
tp = lambda x:list(map(int, x.split('.')))
parser.add_argument('--pyver', type=tp, help='Python version')
parser.add_argument('--rev', help='revision number default:YYYYMMDD')
parser.add_argument('--with-para', action='store_true', help='do make command with option -j')
parser.add_argument('--without-para', action='store_true', help='do make command without option -j')
parser.add_argument('--without-depend', action='store_false', help='Not do depend script')
parser.add_argument('--debug', action='store_true', help='turn on the debug mode')

args = parser.parse_args()

do_command = args.command
if args.manyo_path is not None:
    PREFIX = args.manyo_path
    isOldInstallType = True
elif args.prefix is not None:
    PREFIX = args.prefix
    isOldInstallType = False
else:
    print("Syntax error : option --prefix or --manyo-path is required.")
    sys.exit(-1)
isDebug = args.debug
isWithDepend = args.without_depend
if args.with_para and (not args.without_para):
    isCompPara = True
else:
    isCompPara = False
if args.rev is not None:
    manyo_rev = args.rev
if args.pyver is not None:
    PYVER = "{}.{}".format(args.pyver[0], args.pyver[1])
if args.numth is not None:
    NUM_OF_THREADS = args.numth
if args.other_modules is not None:
    other_mods = args.other_modules.split(",")
    OTHER_MANYO_PAC.extend(other_mods)
    for a_mod in OTHER_MANYO_PAC:
        print("OTHER_MANYO_PAC= {}".format(a_mod))

if args.dist_name is None:
    DIST_NAME = FOUND_DIST_NAME
else:
    DIST_NAME = args.dist_name
    if DIST_NAME == "" or DIST_NAME == "RH" or DIST_NAME == "Ubuntu":
        DIST_NAME = FOUND_DIST_NAME
    elif DIST_NAME == "Ubuntu18":
        DIST_NAME = "Ubuntu16"
    elif DIST_NAME == "Ubuntu22" or DIST_NAME == "Ubuntu24":
        DIST_NAME = "Ubuntu20"
if DIST_NAME == "":
    print("## ERROR : Not found distribution name")
    sys.exit(-1)

print("do_command = {}".format(do_command))
print("isDebug = ", isDebug)
print("isWithDepend = ", isWithDepend)
print("isCompPara = ", isCompPara)
print("manyo_rev = {}".format(manyo_rev))
print("PYVER = {}".format(PYVER))
if isOldInstallType:
    print("MANYO_PATH = {}".format(PREFIX))
else:
    print("PREFIX = {}".format(PREFIX))
print("OTHER_MANYO_PAC = ", OTHER_MANYO_PAC)
print("DIST_NAME = {}".format(DIST_NAME))

if (do_command == "clean"):
    isWithDepend = False

if (do_command == "make"):
    do_command = ""

if os.path.exists(os.path.join(os.getcwd(), "core")):
    isNotManyoCore = False

#############################################
# Check compile target ( manyo-core or not )
#############################################
if isNotManyoCore:
    PACKAGE_NAME = ""
    if os.path.exists(os.path.join(os.getcwd(), "Makefile")):
        fo = open(os.path.join(os.getcwd(), "Makefile"), "r")
        ll = fo.readlines()
        fo.close()
        for aline in ll:
            print(aline)
            if aline.find("PACKAGE_NAME") != -1:
                PACKAGE_NAME = aline.split("=")[1].strip()
                break
    if PACKAGE_NAME == "":
        print("Makefile is invalid. (No PACKAGE_NAME)")
        sys.exit(1)


#########################################################################
# execute make or make install
#############################################
OTHER_MAN_PAC = []
for pac in OTHER_MANYO_PAC:
    OTHER_MAN_PAC.append(pac)

EXT_LIB_PATH = os.path.join(EXT_LIB_ROOT, "lib")
EXT_INC_PATH = os.path.join(EXT_LIB_ROOT, "include")

if DIST_NAME.find("Ubuntu") == 0:
    DIST_PACK = "dist-packages"
else:
    DIST_PACK = "site-packages"

com = ""
if isOldInstallType:
    MANYO_PATH = PREFIX
    com += "export MANYO_PATH={};".format(MANYO_PATH)
    com += "export LIBRARY_PATH=${{MANYO_PATH}}/lib:{};".format(EXT_LIB_PATH)
    com += "export LD_LIBRARY_PATH=${LIBRARY_PATH};"
    com += "export C_INCLUDE_PATH=/usr/include/python{}:".format(PYVER)
    com += ":${MANYO_PATH}/include" + ":{}".format(EXT_INC_PATH)
    for m_pac_name in OTHER_MAN_PAC:
        com += ":${MANYO_PATH}/include/" + "{}".format(m_pac_name)
else:
    MANYO_PATH = os.path.join(PREFIX, "lib", "python{}".format(PYVER), DIST_PACK)
    com += "export MANYO_PATH={};".format(MANYO_PATH)
    com += "export PREFIX={};".format(PREFIX)
    com += "export LIBRARY_PATH=${MANYO_PATH}:" + "{};".format(EXT_LIB_PATH)
    com += "export LD_LIBRARY_PATH=${LIBRARY_PATH};"
    com += "export C_INCLUDE_PATH=/usr/include/python{}:".format(PYVER)
    com += ":${PREFIX}/include/Manyo" + ":{}".format(EXT_INC_PATH)
    for m_pac_name in OTHER_MAN_PAC:
        com += ":${PREFIX}/include/Manyo/" + "{}".format(m_pac_name)

if DIST_NAME.find("Ubuntu20") == 0:
    com += ":/usr/include/hdf5/serial;" ## [inamura 201103 for Ubuntu 20.04]
elif DIST_NAME.find("Ubuntu16") == 0: # for Ubuntu 16 or 18
    com += ":/usr/include/hdf5/serial;" ## [inamura 201103 for Ubuntu 16.04 and 18.04]
else: ## For RH
    com += "; "

com += 'export INC_EXT="'
for m_pac_name in OTHER_MAN_PAC:
    com += "%s "%(m_pac_name)
com += '"; '
com += "export CPLUS_INCLUDE_PATH=${C_INCLUDE_PATH};"

if isWithDepend:
    if isNotManyoCore:
        com += "./depend; "
    else:
        com += 'cd core; ./depend; cd ../MLF; ./depend; cd ../SerialPySrv; ./depend; cd ..; '

if (do_command != "depend"):
    com += 'make {:s} '.format(do_command)
    if isCompPara:
        com += ' -j '
    if DIST_NAME == "":
        DIST_NAME = "Ubuntu"
    com += ' DIST_NAME={:s}'.format(DIST_NAME)
    if PYVER != "":
        com += ' CXX={:s} PyVer={:s} NUM_MULTH={:d}'.format(CXX, PYVER, NUM_OF_THREADS)
    else:
        com += ' CXX={:s} NUM_MULTH={:d}'.format(CXX, NUM_OF_THREADS)
    com += ' MANYO_REV={:d}'.format(manyo_rev)
    if not isOldInstallType and PYVER != "":
        com += " PY_PREFIX={}".format(MANYO_PATH)

print("command = {}".format(com))
if isDebug:
    pass
else:
    retcode = os.system(com)
    if retcode:
        sys.stderr.write("Return code '{}', is not 0.\n".format(retcode))
        raise RuntimeError("Error has been caused in building.")


#########################################################################
# make environ file
#############################################

env_list = []
env_list.append('#!/bin/bash')
env_list.append('export MANYO_PATH={:s}'.format(MANYO_PATH))

if isOldInstallType:
    env_list.append('export LD_LIBRARY_PATH=${MANYO_PATH}/lib:${LD_LIBRARY_PATH}')
else:
    env_list.append('export LD_LIBRARY_PATH=${MANYO_PATH}:${LD_LIBRARY_PATH}')

if isNotManyoCore:
    env_list.append('export PYTHONPATH=${MANYO_PATH}:${MANYO_PATH}/Manyo/' + '%s:'.format(PACKAGE_NAME) + '${PYTHONPATH}')
else:
    env_list.append( 'export PYTHONPATH=${MANYO_PATH}:${PYTHONPATH}' )
env_list.append( 'export PATH={}/bin:$PATH'.format(PREFIX) )


if isDebug:
    for an_env in env_list:
        print(an_env)
else:
    fo = open(os.path.join(os.getcwd(), ENV_FILE_NAME), 'w')
    for ss in env_list:
        fo.write(ss + '\n')
    fo.close()
